"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getApplySemigroup = exports.apT = exports.bindPar = exports.zipRightPar = exports.zipLeftPar = exports.getApComposition = void 0;
/**
 * The `Apply` class provides the `ap` which is used to apply a function to an argument under a type constructor.
 *
 * `Apply` can be used to lift functions of two or more arguments to work on values wrapped with the type constructor
 * `f`.
 *
 * Instances must satisfy the following law in addition to the `Functor` laws:
 *
 * 1. Associative composition: `fbc |> map(bc => ab => a => bc(ab(a))) |> ap(fab) <-> fbc |> ap(fab |> ap(fa))`
 *
 * Formally, `Apply` represents a strong lax semi-monoidal endofunctor.
 *
 * @example
 * import * as O from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * const f = (a: string) => (b: number) => (c: boolean) => a + String(b) + String(c)
 * const fa: O.Option<string> = O.some('s')
 * const fb: O.Option<number> = O.some(1)
 * const fc: O.Option<boolean> = O.some(true)
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     // lift a function
 *     O.some(f),
 *     // apply the first argument
 *     O.ap(fa),
 *     // apply the second argument
 *     O.ap(fb),
 *     // apply the third argument
 *     O.ap(fc)
 *   ),
 *   O.some('s1true')
 * )
 *
 * @since 3.0.0
 */
var function_1 = require("./function");
var semigroup = __importStar(require("./Semigroup"));
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `ap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var getApComposition = function (Applyλ, Applyμ) {
    return function (fa) {
        return (0, function_1.flow)(Applyλ.map(function (gab) { return function (ga) { return Applyμ.ap(ga)(gab); }; }), Applyλ.ap(fa));
    };
};
exports.getApComposition = getApComposition;
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
var zipLeftPar = function (Applyλ) {
    return function (second) {
        return (0, function_1.flow)(Applyλ.map(function (a) { return function () { return a; }; }), Applyλ.ap(second));
    };
};
exports.zipLeftPar = zipLeftPar;
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
var zipRightPar = function (Applyλ) {
    return function (second) {
        return (0, function_1.flow)(Applyλ.map(function () { return function (b) { return b; }; }), Applyλ.ap(second));
    };
};
exports.zipRightPar = zipRightPar;
/**
 * @category combinators
 * @since 3.0.0
 */
var bindPar = function (Applyλ) {
    return function (name, fb) {
        return (0, function_1.flow)(Applyλ.map(function (a) { return function (b) {
            var _a;
            return Object.assign({}, a, (_a = {}, _a[name] = b, _a));
        }; }), Applyλ.ap(fb));
    };
};
exports.bindPar = bindPar;
/**
 * @category combinators
 * @since 3.0.0
 */
var apT = function (Applyλ) {
    return function (fb) {
        return function (fas) {
            return (0, function_1.pipe)(fas, Applyλ.map(function (a) { return function (b) { return __spreadArray(__spreadArray([], a, true), [b], false); }; }), Applyλ.ap(fb));
        };
    };
};
exports.apT = apT;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Lift a semigroup into 'λ', the inner values are combined using the provided `Semigroup`.
 *
 * @since 3.0.0
 */
var getApplySemigroup = function (Applyλ) {
    return function (S) {
        var f = semigroup.reverse(S).combine;
        return {
            combine: function (second) { return function (first) { return (0, function_1.pipe)(first, Applyλ.map(f), Applyλ.ap(second)); }; }
        };
    };
};
exports.getApplySemigroup = getApplySemigroup;
