"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sequenceReadonlyArray = exports.traverseReadonlyArray = exports.traverseReadonlyNonEmptyArray = exports.traverseReadonlyArrayWithIndex = exports.traverseReadonlyNonEmptyArrayWithIndex = exports.apT = exports.tupled = exports.ApT = exports.bindPar = exports.bind = exports.let = exports.bindTo = exports.Do = exports.FlattenableRec = exports.logError = exports.log = exports.FromIO = exports.tap = exports.Monad = exports.Flattenable = exports.Applicative = exports.zipRightPar = exports.zipLeftPar = exports.Apply = exports.Pointed = exports.flap = exports.Functor = exports.flatten = exports.flatMapRec = exports.flatMap = exports.unit = exports.of = exports.ap = exports.map = void 0;
/**
 * ```ts
 * interface IO<A> {
 *   (): A
 * }
 * ```
 *
 * `IO<A>` represents a non-deterministic synchronous computation that can cause side effects, yields a value of
 * type `A` and **never fails**.
 *
 * If you want to represent a synchronous computation that may fail, please see `IOEither`.
 * If you want to represent a synchronous computation that may yield nothing, please see `IOOption`.
 *
 * @since 3.0.0
 */
var apply = __importStar(require("./Apply"));
var flattenable = __importStar(require("./Flattenable"));
var fromIO_ = __importStar(require("./FromIO"));
var function_1 = require("./function");
var functor = __importStar(require("./Functor"));
var _ = __importStar(require("./internal"));
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Functor
 * @since 3.0.0
 */
var map = function (f) { return function (fa) { return function () { return f(fa()); }; }; };
exports.map = map;
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
var ap = function (fa) { return function (fab) { return function () { return fab()(fa()); }; }; };
exports.ap = ap;
/**
 * @category Pointed
 * @since 3.0.0
 */
exports.of = function_1.constant;
/**
 * @since 3.0.0
 */
exports.unit = (0, exports.of)(undefined);
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
var flatMap = function (f) { return function (ma) { return function () { return f(ma())(); }; }; };
exports.flatMap = flatMap;
/**
 * @category FlattenableRec
 * @since 3.0.0
 */
var flatMapRec = function (f) { return function (a) { return function () {
    var e = f(a)();
    while (_.isLeft(e)) {
        e = f(e.left)();
    }
    return e.right;
}; }; };
exports.flatMapRec = flatMapRec;
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flatten = (0, exports.flatMap)(function_1.identity);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = functor.flap(exports.Functor);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Pointed = {
    of: exports.of
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Apply = {
    map: exports.map,
    ap: exports.ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.zipLeftPar = apply.zipLeftPar(exports.Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.zipRightPar = apply.zipRightPar(exports.Apply);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Applicative = {
    map: exports.map,
    ap: exports.ap,
    of: exports.of
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Flattenable = {
    map: exports.map,
    flatMap: exports.flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Monad = {
    map: exports.map,
    of: exports.of,
    flatMap: exports.flatMap
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.tap = flattenable.tap(exports.Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
exports.FromIO = {
    fromIO: function_1.identity
};
// -------------------------------------------------------------------------------------
// logging
// -------------------------------------------------------------------------------------
/**
 * @category logging
 * @since 3.0.0
 */
exports.log = fromIO_.log(exports.FromIO);
/**
 * @category logging
 * @since 3.0.0
 */
exports.logError = fromIO_.logError(exports.FromIO);
/**
 * @category instances
 * @since 3.0.0
 */
exports.FlattenableRec = {
    flatMapRec: exports.flatMapRec
};
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
exports.Do = (0, exports.of)(_.emptyRecord);
/**
 * @since 3.0.0
 */
exports.bindTo = 
/*#__PURE__*/ functor.bindTo(exports.Functor);
var let_ = 
/*#__PURE__*/ functor.let(exports.Functor);
exports.let = let_;
/**
 * @since 3.0.0
 */
exports.bind = 
/*#__PURE__*/ flattenable.bind(exports.Flattenable);
/**
 * @since 3.0.0
 */
exports.bindPar = 
/*#__PURE__*/ apply.bindPar(exports.Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
exports.ApT = (0, exports.of)(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
exports.tupled = functor.tupled(exports.Functor);
/**
 * @since 3.0.0
 */
exports.apT = 
/*#__PURE__*/ apply.apT(exports.Apply);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArrayWithIndex = function (f) {
    return function (as) {
        return function () {
            var out = [f(0, _.head(as))()];
            for (var i = 1; i < as.length; i++) {
                out.push(f(i, as[i])());
            }
            return out;
        };
    };
};
exports.traverseReadonlyNonEmptyArrayWithIndex = traverseReadonlyNonEmptyArrayWithIndex;
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArrayWithIndex = function (f) {
    var g = (0, exports.traverseReadonlyNonEmptyArrayWithIndex)(f);
    return function (as) { return (_.isNonEmpty(as) ? g(as) : exports.ApT); };
};
exports.traverseReadonlyArrayWithIndex = traverseReadonlyArrayWithIndex;
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArray = function (f) {
    return (0, exports.traverseReadonlyNonEmptyArrayWithIndex)(function (_, a) { return f(a); });
};
exports.traverseReadonlyNonEmptyArray = traverseReadonlyNonEmptyArray;
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArray = function (f) {
    return (0, exports.traverseReadonlyArrayWithIndex)(function (_, a) { return f(a); });
};
exports.traverseReadonlyArray = traverseReadonlyArray;
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
exports.sequenceReadonlyArray = 
/*#__PURE__*/ (0, exports.traverseReadonlyArray)(function_1.identity);
