"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sequenceReadonlyArray = exports.traverseReadonlyArray = exports.traverseReadonlyNonEmptyArray = exports.traverseReadonlyArrayWithIndex = exports.traverseReadonlyNonEmptyArrayWithIndex = exports.apT = exports.tupled = exports.bindPar = exports.bind = exports.let = exports.bindTo = exports.execute = exports.evaluate = exports.FromState = exports.Monad = exports.Flattenable = exports.Applicative = exports.zipRightPar = exports.zipLeftPar = exports.Apply = exports.Pointed = exports.flap = exports.Functor = exports.flatten = exports.flatMap = exports.unit = exports.of = exports.ap = exports.map = exports.gets = exports.modify = exports.put = exports.get = void 0;
var apply = __importStar(require("./Apply"));
var flattenable = __importStar(require("./Flattenable"));
var function_1 = require("./function");
var functor = __importStar(require("./Functor"));
var _ = __importStar(require("./internal"));
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Get the current state
 *
 * @category constructors
 * @since 3.0.0
 */
var get = function () {
    return function (s) {
        return [s, s];
    };
};
exports.get = get;
/**
 * Set the state
 *
 * @category constructors
 * @since 3.0.0
 */
var put = function (s) {
    return function () {
        return [s, undefined];
    };
};
exports.put = put;
/**
 * Modify the state by applying a function to the current state
 *
 * @category constructors
 * @since 3.0.0
 */
var modify = function (f) {
    return function (s) {
        return [f(s), undefined];
    };
};
exports.modify = modify;
/**
 * Get a value which depends on the current state
 *
 * @category constructors
 * @since 3.0.0
 */
var gets = function (f) {
    return function (s) {
        return [s, f(s)];
    };
};
exports.gets = gets;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Functor
 * @since 3.0.0
 */
var map = function (f) {
    return function (self) {
        return function (s1) {
            var _a = self(s1), s2 = _a[0], a = _a[1];
            return [s2, f(a)];
        };
    };
};
exports.map = map;
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
var ap = function (fa) {
    return function (self) {
        return function (s1) {
            var _a = self(s1), s2 = _a[0], f = _a[1];
            var _b = fa(s2), s3 = _b[0], a = _b[1];
            return [s3, f(a)];
        };
    };
};
exports.ap = ap;
/**
 * @category Pointed
 * @since 3.0.0
 */
var of = function (a) {
    return function (s) {
        return [s, a];
    };
};
exports.of = of;
/**
 * @since 3.0.0
 */
var unit = function () { return (0, exports.of)(undefined); };
exports.unit = unit;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
var flatMap = function (f) {
    return function (self) {
        return function (s1) {
            var _a = self(s1), s2 = _a[0], a = _a[1];
            return f(a)(s2);
        };
    };
};
exports.flatMap = flatMap;
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flatten = (0, exports.flatMap)(function_1.identity);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = functor.flap(exports.Functor);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Pointed = {
    of: exports.of
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Apply = {
    map: exports.map,
    ap: exports.ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(exports.Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(exports.Apply);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Applicative = {
    map: exports.map,
    ap: exports.ap,
    of: exports.of
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Flattenable = {
    map: exports.map,
    flatMap: exports.flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Monad = {
    map: exports.map,
    of: exports.of,
    flatMap: exports.flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.FromState = {
    fromState: function_1.identity
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Run a computation in the `State` monad, discarding the final state.
 *
 * @since 3.0.0
 */
var evaluate = function (s) {
    return function (ma) {
        return ma(s)[1];
    };
};
exports.evaluate = evaluate;
/**
 * Run a computation in the `State` monad discarding the result.
 *
 * @since 3.0.0
 */
var execute = function (s) {
    return function (ma) {
        return ma(s)[0];
    };
};
exports.execute = execute;
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
exports.bindTo = 
/*#__PURE__*/ functor.bindTo(exports.Functor);
var let_ = 
/*#__PURE__*/ functor.let(exports.Functor);
exports.let = let_;
/**
 * @since 3.0.0
 */
exports.bind = 
/*#__PURE__*/ flattenable.bind(exports.Flattenable);
/**
 * @since 3.0.0
 */
exports.bindPar = 
/*#__PURE__*/ apply.bindPar(exports.Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
exports.tupled = functor.tupled(exports.Functor);
/**
 * @since 3.0.0
 */
exports.apT = 
/*#__PURE__*/ apply.apT(exports.Apply);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArrayWithIndex = function (f) {
    return function (as) {
        return function (s) {
            var _a = f(0, _.head(as))(s), s2 = _a[0], b = _a[1];
            var bs = [b];
            var out = s2;
            for (var i = 1; i < as.length; i++) {
                var _b = f(i, as[i])(out), s2_1 = _b[0], b_1 = _b[1];
                bs.push(b_1);
                out = s2_1;
            }
            return [out, bs];
        };
    };
};
exports.traverseReadonlyNonEmptyArrayWithIndex = traverseReadonlyNonEmptyArrayWithIndex;
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArrayWithIndex = function (f) {
    var g = (0, exports.traverseReadonlyNonEmptyArrayWithIndex)(f);
    return function (as) {
        return _.isNonEmpty(as) ? g(as) : (0, exports.of)(_.emptyReadonlyArray);
    };
};
exports.traverseReadonlyArrayWithIndex = traverseReadonlyArrayWithIndex;
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArray = function (f) {
    return (0, exports.traverseReadonlyNonEmptyArrayWithIndex)(function (_, a) { return f(a); });
};
exports.traverseReadonlyNonEmptyArray = traverseReadonlyNonEmptyArray;
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArray = function (f) {
    return (0, exports.traverseReadonlyArrayWithIndex)(function (_, a) { return f(a); });
};
exports.traverseReadonlyArray = traverseReadonlyArray;
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
exports.sequenceReadonlyArray = 
/*#__PURE__*/ (0, exports.traverseReadonlyArray)(function_1.identity);
