"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sequenceReadonlyArraySeq = exports.traverseReadonlyArraySeq = exports.traverseReadonlyNonEmptyArraySeq = exports.traverseReadonlyArrayWithIndexSeq = exports.traverseReadonlyNonEmptyArrayWithIndexSeq = exports.sequenceReadonlyArray = exports.traverseReadonlyArray = exports.traverseReadonlyNonEmptyArray = exports.traverseReadonlyArrayWithIndex = exports.traverseReadonlyNonEmptyArrayWithIndex = exports.apT = exports.tupled = exports.ApT = exports.bindPar = exports.bind = exports.let = exports.bindTo = exports.Do = exports.never = exports.FromTask = exports.flatMapIOK = exports.fromIOK = exports.logError = exports.log = exports.FromIO = exports.Monad = exports.tap = exports.ApplicativeSeq = exports.ApplySeq = exports.Flattenable = exports.ApplicativePar = exports.zipRightPar = exports.zipLeftPar = exports.ApplyPar = exports.Pointed = exports.flap = exports.Functor = exports.getRaceMonoid = exports.flatten = exports.flatMap = exports.unit = exports.of = exports.ap = exports.map = exports.delay = exports.fromIO = void 0;
var apply = __importStar(require("./Apply"));
var flattenable = __importStar(require("./Flattenable"));
var fromIO_ = __importStar(require("./FromIO"));
var function_1 = require("./function");
var functor = __importStar(require("./Functor"));
var _ = __importStar(require("./internal"));
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
var fromIO = function (ma) { return function () { return Promise.resolve().then(ma); }; };
exports.fromIO = fromIO;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Creates a task that will complete after a time delay
 *
 * @example
 * import { pipe } from 'fp-ts/function'
 * import * as T from 'fp-ts/Task'
 * import { takeRight } from 'fp-ts/ReadonlyArray'
 *
 * async function test() {
 *   const log: Array<string> = []
 *   const append = (message: string): T.Task<void> =>
 *     T.fromIO(() => {
 *       log.push(message)
 *     })
 *   const fa = append('a')
 *   const fb = T.delay(20)(append('b'))
 *   const fc = T.delay(10)(append('c'))
 *   const fd = append('d')
 *   await pipe(T.ApT, T.apT(fa), T.apT(fb), T.apT(fc), T.apT(fd))()
 *   assert.deepStrictEqual(takeRight(2)(log), ['c', 'b'])
 * }
 *
 * test()
 *
 * @category combinators
 * @since 3.0.0
 */
var delay = function (millis) {
    return function (ma) {
        return function () {
            return new Promise(function (resolve) {
                setTimeout(function () {
                    // tslint:disable-next-line: no-floating-promises
                    Promise.resolve().then(ma).then(resolve);
                }, millis);
            });
        };
    };
};
exports.delay = delay;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Functor
 * @since 3.0.0
 */
var map = function (f) { return function (fa) { return function () {
    return Promise.resolve().then(fa).then(f);
}; }; };
exports.map = map;
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
var ap = function (fa) { return function (fab) { return function () {
    return Promise.all([Promise.resolve().then(fab), Promise.resolve().then(fa)]).then(function (_a) {
        var f = _a[0], a = _a[1];
        return f(a);
    });
}; }; };
exports.ap = ap;
/**
 * @category Pointed
 * @since 3.0.0
 */
var of = function (a) { return function () { return Promise.resolve(a); }; };
exports.of = of;
/**
 * @since 3.0.0
 */
exports.unit = (0, exports.of)(undefined);
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
var flatMap = function (f) { return function (ma) { return function () {
    return Promise.resolve()
        .then(ma)
        .then(function (a) { return f(a)(); });
}; }; };
exports.flatMap = flatMap;
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flatten = (0, exports.flatMap)(function_1.identity);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * Monoid returning the first completed task.
 *
 * Note: uses `Promise.race` internally.
 *
 * @example
 * import * as T from 'fp-ts/Task'
 * import { pipe } from 'fp-ts/function'
 *
 * async function test() {
 *   const S = T.getRaceMonoid<string>()
 *   const fa = T.delay(20)(T.of('a'))
 *   const fb = T.delay(10)(T.of('b'))
 *   assert.deepStrictEqual(await pipe(fa, S.combine(fb))(), 'b')
 * }
 *
 * test()
 *
 * @category instances
 * @since 3.0.0
 */
var getRaceMonoid = function () { return ({
    combine: function (second) { return function (first) { return function () { return Promise.race([Promise.resolve().then(first), Promise.resolve().then(second)]); }; }; },
    empty: exports.never
}); };
exports.getRaceMonoid = getRaceMonoid;
/**
 * @category instances
 * @since 3.0.0
 */
exports.Functor = {
    map: exports.map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.flap = functor.flap(exports.Functor);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Pointed = {
    of: exports.of
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.ApplyPar = {
    map: exports.map,
    ap: exports.ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(exports.ApplyPar);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(exports.ApplyPar);
/**
 * @category instances
 * @since 3.0.0
 */
exports.ApplicativePar = {
    map: exports.map,
    ap: exports.ap,
    of: exports.of
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Flattenable = {
    map: exports.map,
    flatMap: exports.flatMap
};
var apSeq = /*#__PURE__*/ flattenable.ap(exports.Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
exports.ApplySeq = {
    map: exports.map,
    ap: apSeq
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.ApplicativeSeq = {
    map: exports.map,
    ap: apSeq,
    of: exports.of
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
exports.tap = 
/*#__PURE__*/ flattenable.tap(exports.Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
exports.Monad = {
    map: exports.map,
    of: exports.of,
    flatMap: exports.flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.FromIO = {
    fromIO: exports.fromIO
};
// -------------------------------------------------------------------------------------
// logging
// -------------------------------------------------------------------------------------
/**
 * @category logging
 * @since 3.0.0
 */
exports.log = fromIO_.log(exports.FromIO);
/**
 * @category logging
 * @since 3.0.0
 */
exports.logError = fromIO_.logError(exports.FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
exports.fromIOK = 
/*#__PURE__*/ fromIO_.fromIOK(exports.FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
exports.flatMapIOK = fromIO_.flatMapIOK(exports.FromIO, exports.Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
exports.FromTask = {
    fromIO: exports.fromIO,
    fromTask: function_1.identity
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * A `Task` that never completes.
 *
 * @since 3.0.0
 */
var never = function () { return new Promise(function () { return undefined; }); };
exports.never = never;
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
exports.Do = (0, exports.of)(_.emptyRecord);
/**
 * @since 3.0.0
 */
exports.bindTo = 
/*#__PURE__*/ functor.bindTo(exports.Functor);
var let_ = 
/*#__PURE__*/ functor.let(exports.Functor);
exports.let = let_;
/**
 * @since 3.0.0
 */
exports.bind = 
/*#__PURE__*/ flattenable.bind(exports.Flattenable);
/**
 * @since 3.0.0
 */
exports.bindPar = 
/*#__PURE__*/ apply.bindPar(exports.ApplyPar);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
exports.ApT = (0, exports.of)(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
exports.tupled = functor.tupled(exports.Functor);
/**
 * @since 3.0.0
 */
exports.apT = 
/*#__PURE__*/ apply.apT(exports.ApplyPar);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
// --- Par ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplyPar)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArrayWithIndex = function (f) {
    return function (as) {
        return function () {
            return Promise.all(as.map(function (a, i) { return Promise.resolve().then(function () { return f(i, a)(); }); }));
        };
    };
};
exports.traverseReadonlyNonEmptyArrayWithIndex = traverseReadonlyNonEmptyArrayWithIndex;
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArrayWithIndex = function (f) {
    var g = (0, exports.traverseReadonlyNonEmptyArrayWithIndex)(f);
    return function (as) { return (_.isNonEmpty(as) ? g(as) : exports.ApT); };
};
exports.traverseReadonlyArrayWithIndex = traverseReadonlyArrayWithIndex;
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplyPar)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArray = function (f) {
    return (0, exports.traverseReadonlyNonEmptyArrayWithIndex)(function (_, a) { return f(a); });
};
exports.traverseReadonlyNonEmptyArray = traverseReadonlyNonEmptyArray;
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativePar)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArray = function (f) {
    return (0, exports.traverseReadonlyArrayWithIndex)(function (_, a) { return f(a); });
};
exports.traverseReadonlyArray = traverseReadonlyArray;
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativePar)`.
 *
 * @since 3.0.0
 */
exports.sequenceReadonlyArray = 
/*#__PURE__*/ (0, exports.traverseReadonlyArray)(function_1.identity);
// --- Seq ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplySeq)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArrayWithIndexSeq = function (f) {
    return function (as) {
        return function () {
            return _.tail(as).reduce(function (acc, a, i) {
                return acc.then(function (bs) {
                    return Promise.resolve()
                        .then(f(i + 1, a))
                        .then(function (b) {
                        bs.push(b);
                        return bs;
                    });
                });
            }, Promise.resolve()
                .then(f(0, _.head(as)))
                .then(_.singleton));
        };
    };
};
exports.traverseReadonlyNonEmptyArrayWithIndexSeq = traverseReadonlyNonEmptyArrayWithIndexSeq;
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArrayWithIndexSeq = function (f) {
    var g = (0, exports.traverseReadonlyNonEmptyArrayWithIndexSeq)(f);
    return function (as) { return (_.isNonEmpty(as) ? g(as) : exports.ApT); };
};
exports.traverseReadonlyArrayWithIndexSeq = traverseReadonlyArrayWithIndexSeq;
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplySeq)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyNonEmptyArraySeq = function (f) {
    return (0, exports.traverseReadonlyNonEmptyArrayWithIndexSeq)(function (_, a) { return f(a); });
};
exports.traverseReadonlyNonEmptyArraySeq = traverseReadonlyNonEmptyArraySeq;
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
var traverseReadonlyArraySeq = function (f) {
    return (0, exports.traverseReadonlyArrayWithIndexSeq)(function (_, a) { return f(a); });
};
exports.traverseReadonlyArraySeq = traverseReadonlyArraySeq;
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
exports.sequenceReadonlyArraySeq = 
/*#__PURE__*/ (0, exports.traverseReadonlyArraySeq)(function_1.identity);
