"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MonoidProduct = exports.MonoidSum = exports.SemigroupProduct = exports.SemigroupSum = exports.MagmaSub = exports.Show = exports.Field = exports.Bounded = exports.Ord = exports.Eq = exports.isNumber = void 0;
var eq = __importStar(require("./Eq"));
// -------------------------------------------------------------------------------------
// refinements
// -------------------------------------------------------------------------------------
/**
 * @category refinements
 * @since 3.0.0
 */
var isNumber = function (u) { return typeof u === 'number'; };
exports.isNumber = isNumber;
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
exports.Eq = eq.EqStrict;
/**
 * @category instances
 * @since 3.0.0
 */
exports.Ord = {
    compare: function (second) { return function (first) { return first < second ? -1 : first > second ? 1 : 0; }; }
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Bounded = {
    compare: exports.Ord.compare,
    top: Infinity,
    bottom: -Infinity
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Field = {
    add: function (second) { return function (first) { return first + second; }; },
    zero: 0,
    mul: function (second) { return function (first) { return first * second; }; },
    one: 1,
    sub: function (second) { return function (first) { return first - second; }; },
    degree: function () { return 1; },
    div: function (second) { return function (first) { return first / second; }; },
    mod: function (second) { return function (first) { return first % second; }; }
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.Show = {
    show: function (a) { return JSON.stringify(a); }
};
/**
 * @category instances
 * @since 3.0.0
 */
exports.MagmaSub = {
    combine: exports.Field.sub
};
/**
 * `number` semigroup under addition.
 *
 * @example
 * import { SemigroupSum } from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(2, SemigroupSum.combine(3)), 5)
 *
 * @category instances
 * @since 3.0.0
 */
exports.SemigroupSum = {
    combine: function (second) { return function (first) { return first + second; }; }
};
/**
 * `number` semigroup under multiplication.
 *
 * @example
 * import { SemigroupProduct } from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(2, SemigroupProduct.combine(3)), 6)
 *
 * @category instances
 * @since 3.0.0
 */
exports.SemigroupProduct = {
    combine: function (second) { return function (first) { return first * second; }; }
};
/**
 * `number` monoid under addition.
 *
 * The `empty` value is `0`.
 *
 * @category instances
 * @since 3.0.0
 */
exports.MonoidSum = {
    combine: exports.SemigroupSum.combine,
    empty: 0
};
/**
 * `number` monoid under multiplication.
 *
 * The `empty` value is `1`.
 *
 * @category instances
 * @since 3.0.0
 */
exports.MonoidProduct = {
    combine: exports.SemigroupProduct.combine,
    empty: 1
};
