import { constVoid, flow, pipe } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as writer from './Writer';
// -------------------------------------------------------------------------------------
// defaults
// -------------------------------------------------------------------------------------
/**
 * Return a default `compact` implementation from `Functor` and `separate`.
 *
 * @category defaults
 * @since 3.0.0
 */
export const getDefaultCompact = (F) => (separate) => {
    return flow(F.map(_.fromOption(constVoid)), separate, writer.snd);
};
/**
 * Return a default `separate` implementation from `Functor` and `compact`.
 *
 * @category defaults
 * @since 3.0.0
 */
export function getDefaultSeparate(F) {
    return (compact) => (fe) => [pipe(fe, F.map(_.getLeft), compact), pipe(fe, F.map(_.getRight), compact)];
}
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `compact` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export function getCompactComposition(F, G) {
    return F.map(G.compact);
}
/**
 * `separate` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export function getSeparateComposition(F, C, G) {
    const compactFC = getCompactComposition(F, C);
    const mapFG = functor.getMapComposition(F, G);
    return (fge) => [pipe(fge, mapFG(_.getLeft), compactFC), pipe(fge, mapFG(_.getRight), compactFC)];
}
