import * as eq from './Eq';
import { unsafeCoerce } from './function';
import * as functor from './Functor';
import * as ord from './Ord';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const make = (s) => unsafeCoerce({
    value: s
});
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const contramap = () => unsafeCoerce;
/**
 * @since 3.0.0
 */
export const mapLeft = (f) => (self) => make(f(self.value));
/**
 * @since 3.0.0
 */
export const mapBoth = unsafeCoerce(mapLeft);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = eq.contramap((c) => c.value);
/**
 * @category instances
 * @since 3.0.0
 */
export const getOrd = ord.contramap((c) => c.value);
/**
 * @category instances
 * @since 3.0.0
 */
export const getBounded = (B) => ({
    compare: getOrd(B).compare,
    top: make(B.top),
    bottom: make(B.bottom)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (S) => ({
    show: (c) => `make(${S.show(c.value)})`
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getSemigroup = (S) => ({
    combine: (that) => (self) => make(S.combine(that.value)(self.value))
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonoid = (M) => ({
    combine: getSemigroup(M).combine,
    empty: make(M.empty)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getSemiring = (S) => ({
    add: (that) => (self) => make(S.add(that.value)(self.value)),
    mul: (that) => (self) => make(S.mul(that.value)(self.value)),
    one: make(S.one),
    zero: make(S.zero)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getRing = (R) => {
    const S = getSemiring(R);
    return {
        add: S.add,
        mul: S.mul,
        one: S.one,
        zero: S.zero,
        sub: (that) => (self) => make(R.sub(that.value)(self.value))
    };
};
// TODO Field
/**
 * @category instances
 * @since 3.0.0
 */
export const getHeytingAlgebra = (H) => {
    return {
        implies: (that) => (self) => make(H.implies(that.value)(self.value)),
        not: (c) => make(H.not(c.value)),
        join: (that) => (self) => make(H.join(that.value)(self.value)),
        meet: (that) => (self) => make(H.meet(that.value)(self.value)),
        one: make(H.one),
        zero: make(H.zero)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getBooleanAlgebra = getHeytingAlgebra;
/**
 * @category Functor
 * @since 3.0.0
 */
export const map = () => unsafeCoerce;
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Contravariant = {
    contramap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    mapBoth
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (S) => ({
    map,
    ap: (fa) => (self) => make(S.combine(fa.value)(self.value))
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (M) => {
    const A = getApply(M);
    const empty = make(M.empty);
    return {
        map: A.map,
        ap: A.ap,
        of: () => empty
    };
};
