import { flow, pipe } from './function';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const filter = (F) => (predicate) => F.filterMap((b) => (predicate(b) ? _.some(b) : _.none));
/**
 * @category combinators
 * @since 3.0.0
 */
export const partition = (F) => (predicate) => F.partitionMap((b) => (predicate(b) ? _.right(b) : _.left(b)));
/**
 * `filterMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const getFilterMapComposition = (F, G) => {
    return (f) => F.map(G.filterMap(f));
};
/**
 * `partitionMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const getPartitionMapComposition = (F, G) => {
    const filterMap = getFilterMapComposition(F, G);
    return (f) => (self) => [pipe(self, filterMap(flow(f, _.getLeft))), pipe(self, filterMap(flow(f, _.getRight)))];
};
