import { flow, pipe } from './function';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// defaults
// -------------------------------------------------------------------------------------
/**
 * Return a `partitionMapWithEffect` implementation from `Traversable` and `Compactable`.
 *
 * @category defaults
 * @since 3.0.0
 */
export function getDefaultPartitionMapWithEffect(T, C) {
    return (F) => {
        const traverseF = T.traverse(F);
        return (f) => {
            return flow(traverseF(f), F.map(C.separate));
        };
    };
}
/**
 * Return a `filterMapWithEffect` implementation from `Traversable` and `Compactable`.
 *
 * @category defaults
 * @since 3.0.0
 */
export function getDefaultFilterMapWithEffect(T, C) {
    return (F) => {
        const traverseF = T.traverse(F);
        return (f) => {
            return flow(traverseF(f), F.map(C.compact));
        };
    };
}
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Filter values inside a `F` context.
 *
 * @category combinators
 * @since 3.0.0
 */
export const filterWithEffect = (FilterableWithEffectμ) => (Applicativeλ) => {
    const filterMapWithEffectλ = FilterableWithEffectμ.filterMapWithEffect(Applicativeλ);
    return (predicateK) => {
        return filterMapWithEffectλ((b) => pipe(predicateK(b), Applicativeλ.map((bool) => (bool ? _.some(b) : _.none))));
    };
};
/**
 * Partition values inside a `F` context.
 *
 * @category combinators
 * @since 3.0.0
 */
export const partitionWithEffect = (FilterableWithEffectμ) => (Applicativeλ) => {
    const partitionMapWithEffectλ = FilterableWithEffectμ.partitionMapWithEffect(Applicativeλ);
    return (predicateK) => {
        return partitionMapWithEffectλ((b) => pipe(predicateK(b), Applicativeλ.map((bool) => (bool ? _.right(b) : _.left(b)))));
    };
};
