import { pipe, flow } from './function';
import * as _ from './internal';
import { not } from './Predicate';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromOption = (F) => (onNone) => {
    const fromOption = _.fromOption(onNone);
    return (ma) => F.fromEither(fromOption(ma));
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = (F) => (predicate, onFalse) => (b) => F.fromEither(predicate(b) ? _.right(b) : _.left(onFalse(b)));
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = (F) => {
    return (f, onNone) => (...a) => {
        return F.fromEither(_.fromOptionOrElse(f(...a), () => onNone(...a)));
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapOptionK = (F, M) => {
    const fromOptionKF = fromOptionK(F);
    return (f, onNone) => {
        const from = fromOptionKF(f, onNone);
        return (self) => {
            return pipe(self, M.flatMap(from));
        };
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = (F) => (f) => (...a) => F.fromEither(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapEitherK = (F, M) => {
    const fromEitherKF = fromEitherK(F);
    return (f) => (self) => {
        return pipe(self, M.flatMap(fromEitherKF(f)));
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterMap = (F, M) => (f, onNone) => {
    return M.flatMap((a) => {
        const ob = f(a);
        return F.fromEither(_.isNone(ob) ? _.left(onNone(a)) : _.right(ob.value));
    });
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const partitionMap = (F, M) => (f, onEmpty) => (self) => {
    const filterMapFM = filterMap(F, M);
    return [pipe(self, filterMapFM(flow(f, _.getLeft), onEmpty)), pipe(self, filterMapFM(flow(f, _.getRight), onEmpty))];
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const filter = (F, M) => (predicate, onFalse) => {
    return M.flatMap((b) => F.fromEither(predicate(b) ? _.right(b) : _.left(onFalse(b))));
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const partition = (F, M) => (predicate, onFalse) => (self) => {
    const filterFM = filter(F, M);
    return [pipe(self, filterFM(not(predicate), onFalse)), pipe(self, filterFM(predicate, onFalse))];
};
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = (F) => (onNullable) => (a) => {
    return F.fromEither(_.fromNullableOrElse(a, onNullable));
};
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = (F) => {
    const fromNullableF = fromNullable(F);
    return (onNullable) => {
        const fromNullable = fromNullableF(onNullable);
        return (f) => (...a) => {
            return fromNullable(f(...a));
        };
    };
};
/**
 * @category interop
 * @since 3.0.0
 */
export const flatMapNullableK = (F, M) => {
    const fromNullableKM = fromNullableK(F);
    return (onNullable) => {
        const fromNullable = fromNullableKM(onNullable);
        return (f) => (self) => {
            return pipe(self, M.flatMap(fromNullable(f)));
        };
    };
};
