import * as monoidK from './MonoidK';
import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as compactable from './Compactable';
import * as filterable from './Filterable';
import * as fromOption_ from './FromOption';
import * as fromEither_ from './FromEither';
import * as fromIO_ from './FromIO';
import { flow, identity, SK } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as io from './IO';
import * as option from './Option';
import * as optionT from './OptionT';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const some = /*#__PURE__*/ optionT.some(io.Pointed);
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = io.of;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = /*#__PURE__*/ optionT.fromEither(io.Pointed);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = /*#__PURE__*/ optionT.fromF(io.Functor);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = /*#__PURE__*/ io.map(option.fromEither);
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = 
/*#__PURE__*/ optionT.match(io.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchWithEffect = /*#__PURE__*/ optionT.matchWithEffect(io.Flattenable);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElse = /*#__PURE__*/ optionT.getOrElse(io.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseE = 
/*#__PURE__*/ optionT.getOrElseE(io.Monad);
/**
 * @category destructors
 * @since 3.0.0
 */
export const toUndefined = io.map(option.toUndefined);
/**
 * @category destructors
 * @since 3.0.0
 */
export const toNullable = io.map(option.toNullable);
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = /*#__PURE__*/ optionT.map(io.Functor);
/**
 * @category Apply
 * @since 3.0.0
 */
export const ap = /*#__PURE__*/ optionT.ap(io.Apply);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = some;
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = 
/*#__PURE__*/ optionT.flatMap(io.Monad);
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ flatMap(identity);
/**
 * @category SemigroupK
 * @since 3.0.0
 */
export const combineK = 
/*#__PURE__*/ optionT.combineK(io.Monad);
/**
 * @category MonoidK
 * @since 3.0.0
 */
export const emptyK = /*#__PURE__*/ optionT.emptyK(io.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const none = /*#__PURE__*/ emptyK();
/**
 * @category Compactable
 * @since 3.0.0
 */
export const compact = 
/*#__PURE__*/ compactable.getCompactComposition(io.Functor, option.Compactable);
/**
 * @category Compactable
 * @since 3.0.0
 */
export const separate = 
/*#__PURE__*/ compactable.getSeparateComposition(io.Functor, option.Compactable, option.Functor);
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filterMap = 
/*#__PURE__*/ filterable.getFilterMapComposition(io.Functor, option.Filterable);
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partitionMap = /*#__PURE__*/ filterable.getPartitionMapComposition(io.Functor, option.Filterable);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(Apply);
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export const tap = 
/*#__PURE__*/ flattenable.tap(Flattenable);
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export const tapError = 
/*#__PURE__*/ optionT.tapNone(io.Monad);
/**
 * @category instances
 * @since 3.0.0
 */
export const SemigroupK = {
    combineK
};
/**
 * @category instances
 * @since 3.0.0
 */
export const MonoidK = {
    combineK,
    emptyK
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const guard = /*#__PURE__*/ monoidK.guard(MonoidK, Pointed);
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Compactable = {
    compact,
    separate
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Filterable = {
    filterMap,
    partitionMap
};
/**
 * @since 3.0.0
 */
export const filter = /*#__PURE__*/ filterable.filter(Filterable);
/**
 * @since 3.0.0
 */
export const partition = /*#__PURE__*/ filterable.partition(Filterable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
// -------------------------------------------------------------------------------------
// logging
// -------------------------------------------------------------------------------------
/**
 * @category logging
 * @since 3.0.0
 */
export const log = /*#__PURE__*/ fromIO_.log(FromIO);
/**
 * @category logging
 * @since 3.0.0
 */
export const logError = /*#__PURE__*/ fromIO_.logError(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = 
/*#__PURE__*/ fromIO_.fromIOK(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOK = 
/*#__PURE__*/ fromIO_.flatMapIOK(FromIO, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromOption = {
    fromOption
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/ fromOption_.fromPredicate(FromOption);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromRefinement = 
/*#__PURE__*/ fromOption_.fromRefinement(FromOption);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = 
/*#__PURE__*/ fromOption_.fromOptionK(FromOption);
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = /*#__PURE__*/ fromOption_.fromNullable(FromOption);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = /*#__PURE__*/ fromOption_.fromNullableK(FromOption);
/**
 * @category interop
 * @since 3.0.0
 */
export const flatMapNullableK = /*#__PURE__*/ fromOption_.flatMapNullableK(FromOption, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = /*#__PURE__*/ fromEither_.fromEitherK(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapEitherK = 
/*#__PURE__*/ fromEither_.flatMapEitherK(FromEither, Flattenable);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(io.traverseReadonlyNonEmptyArrayWithIndex(f), io.map(option.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (f) => {
    return traverseReadonlyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = 
/*#__PURE__*/ traverseReadonlyArray(identity);
