import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as flattenableRec from './FlattenableRec';
import { flow, identity } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as traversable from './Traversable';
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = identity;
/**
 * @category combinators
 * @since 3.0.0
 */
export const map = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ functor.flap(Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const of = identity;
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const ap = (fa) => (fab) => fab(fa);
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatMap = (f) => (ma) => f(ma);
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ flatMap(identity);
/**
 * @since 3.0.0
 */
export const flatMapRec = flattenableRec.tailRec;
/**
 * @category instances
 * @since 3.0.0
 */
export const FlattenableRec = {
    flatMapRec
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const extend = (f) => (wa) => f(wa);
/**
 * @category combinators
 * @since 3.0.0
 */
export const duplicate = /*#__PURE__*/ extend(identity);
/**
 * @category instances
 * @since 3.0.0
 */
export const Extendable = {
    map,
    extend
};
/**
 * @since 3.0.0
 */
export const extract = identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map,
    extend,
    extract
};
/**
 * @since 3.0.0
 */
export const reduce = (b, f) => (fa) => f(b, fa);
/**
 * @since 3.0.0
 */
export const foldMap = () => (f) => (fa) => f(fa);
/**
 * @since 3.0.0
 */
export const reduceRight = (b, f) => (fa) => f(fa, b);
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const traverse = (F) => (f) => flow(f, F.map(identity));
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    traverse
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const sequence = 
/*#__PURE__*/ traversable.getDefaultSequence(traverse);
/**
 * @category combinators
 * @since 3.0.0
 */
export const combineK = () => identity;
/**
 * @category instances
 * @since 3.0.0
 */
export const SemigroupK = {
    combineK
};
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @category do notation
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @category do notation
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/ functor.bindTo(Functor);
const let_ = /*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @category do notation
 * @since 3.0.0
 */
let_ as let };
/**
 * @category do notation
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @category do notation
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(Apply);
/**
 * @category do notation
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
/**
 * @category do notation
 * @since 3.0.0
 */
export const tupled = /*#__PURE__*/ functor.tupled(Functor);
/**
 * @category do notation
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/ apply.apT(Apply);
