/**
 * @since 3.0.0
 */
import * as apply from './Apply';
import { constant, flow, pipe } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as option from './Option';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const some = (F) => (a) => F.of(_.some(a));
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export function fromF(F) {
    return F.map(_.some);
}
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = (F) => (e) => F.of(option.fromEither(e));
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function match(F) {
    return flow(option.match, F.map);
}
/**
 * @since 3.0.0
 */
export const matchWithEffect = (M) => (onNone, onSome) => {
    return M.flatMap(option.match(onNone, onSome));
};
/**
 * @since 3.0.0
 */
export const getOrElse = (F) => (onNone) => {
    return F.map(option.getOrElse(onNone));
};
/**
 * @since 3.0.0
 */
export const getOrElseE = (M) => (onNone) => (self) => {
    return pipe(self, M.flatMap(option.match(onNone, M.of)));
};
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export const tapNone = (M) => {
    const someM = some(M);
    return (onNone) => (self) => {
        return pipe(self, M.flatMap(option.match(flow(onNone, M.map(() => _.none)), someM)));
    };
};
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function map(F) {
    return functor.getMapComposition(F, option.Functor);
}
/**
 * @since 3.0.0
 */
export const ap = (F) => {
    return apply.getApComposition(F, option.Apply);
};
/**
 * @since 3.0.0
 */
export const flatMap = (M) => (f) => (self) => {
    return pipe(self, M.flatMap(option.match(() => emptyK(M)(), f)));
};
/**
 * @since 3.0.0
 */
export const combineK = (M) => {
    const someM = some(M);
    return (second) => (self) => {
        return pipe(self, M.flatMap(option.match(second, someM)));
    };
};
/**
 * @since 3.0.0
 */
export function emptyK(F) {
    return constant(F.of(_.none));
}
