import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as compactable from './Compactable';
import * as either from './Either';
import * as eitherT from './EitherT';
import * as fromEither_ from './FromEither';
import * as fromReader_ from './FromReader';
import { flow, identity, SK } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as reader from './Reader';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = /*#__PURE__*/ eitherT.left(reader.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = /*#__PURE__*/ eitherT.right(reader.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightReader = /*#__PURE__*/ eitherT.rightF(reader.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftReader = /*#__PURE__*/ eitherT.leftF(reader.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const asksReaderEither = reader.asksReader;
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = reader.of;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReader = rightReader;
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = /*#__PURE__*/ eitherT.match(reader.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchWithEffect = /*#__PURE__*/ eitherT.matchWithEffect(reader.Monad);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElse = 
/*#__PURE__*/ eitherT.getOrElse(reader.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseE = /*#__PURE__*/ eitherT.getOrElseE(reader.Monad);
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const toUnion = /*#__PURE__*/ eitherT.toUnion(reader.Functor);
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = reader.local;
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElse = /*#__PURE__*/ eitherT.orElse(reader.Monad);
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = /*#__PURE__*/ eitherT.swap(reader.Functor);
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/ eitherT.map(reader.Functor);
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapBoth = /*#__PURE__*/ eitherT.mapBoth(reader.Functor);
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapError = 
/*#__PURE__*/ eitherT.mapLeft(reader.Functor);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/ eitherT.ap(reader.Apply);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = /*#__PURE__*/ eitherT.flatMap(reader.Monad);
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ flatMap(identity);
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category SemigroupK
 * @since 3.0.0
 */
export const combineK = /*#__PURE__*/ eitherT.combineK(reader.Monad);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * The default [`Applicative`](#applicative) instance returns the first error, if you want to
 * get all errors you need to provide an way to combine them via a `Semigroup`.
 *
 * See [`getValidatedApplicative`](./Either.ts.html#getvalidatedapplicative).
 *
 * @category instances
 * @since 3.0.0
 */
export const getValidatedApplicative = (S) => ({
    map,
    ap: apply.getApComposition(reader.Apply, either.getValidatedApplicative(S)),
    of
});
/**
 * The default [`SemigroupK`](#semigroupk) instance returns the last error, if you want to
 * get all errors you need to provide an way to combine them via a `Semigroup`.
 *
 * @category instances
 * @since 3.0.0
 */
export const getValidatedSemigroupK = (S) => {
    return {
        combineK: eitherT.getValidatedCombineK(reader.Monad, S)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getCompactable = (M) => {
    const C = either.getCompactable(M);
    const F = { map: either.map };
    return {
        compact: compactable.getCompactComposition(reader.Functor, C),
        separate: compactable.getSeparateComposition(reader.Functor, C, F)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFilterable = (M) => {
    return {
        partitionMap: (f) => partitionMap(f, () => M.empty),
        filterMap: (f) => filterMap(f, () => M.empty)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export const tap = 
/*#__PURE__*/ flattenable.tap(Flattenable);
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export const tapError = /*#__PURE__*/ eitherT.tapLeft(reader.Monad);
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    mapBoth
};
/**
 * @category instances
 * @since 3.0.0
 */
export const SemigroupK = {
    combineK
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromReader = {
    fromReader
};
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export const ask = /*#__PURE__*/ fromReader_.ask(FromReader);
/**
 * Projects a value from the global context in a `ReaderEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const asks = /*#__PURE__*/ fromReader_.asks(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderK = /*#__PURE__*/ fromReader_.fromReaderK(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapReaderK = /*#__PURE__*/ fromReader_.flatMapReaderK(FromReader, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/ fromEither_.fromOption(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = /*#__PURE__*/ fromEither_.fromOptionK(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapOptionK = /*#__PURE__*/ fromEither_.flatMapOptionK(FromEither, Flattenable);
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = /*#__PURE__*/ fromEither_.fromPredicate(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filter = /*#__PURE__*/ fromEither_.filter(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterMap = /*#__PURE__*/ fromEither_.filterMap(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partition = /*#__PURE__*/ fromEither_.partition(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partitionMap = 
/*#__PURE__*/ fromEither_.partitionMap(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = /*#__PURE__*/ fromEither_.fromEitherK(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapEitherK = /*#__PURE__*/ fromEither_.flatMapEitherK(FromEither, Flattenable);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = 
/*#__PURE__*/ fromEither_.fromNullable(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = /*#__PURE__*/ fromEither_.fromNullableK(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const flatMapNullableK = 
/*#__PURE__*/ fromEither_.flatMapNullableK(FromEither, Flattenable);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = 
/*#__PURE__*/ functor.tupled(Functor);
/**
 * @since 3.0.0
 */
export const apT = /*#__PURE__*/ apply.apT(Apply);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Make sure that a resource is cleaned up in the event of an exception (\*). The release action is called regardless of
 * whether the body action throws (\*) or returns.
 *
 * (\*) i.e. returns a `Left`
 *
 * @since 3.0.0
 */
export const bracket = /*#__PURE__*/ eitherT.bracket(reader.Monad);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(reader.traverseReadonlyNonEmptyArrayWithIndex(f), reader.map(either.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (f) => {
    return traverseReadonlyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = /*#__PURE__*/ traverseReadonlyArray(identity);
