import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as fromIO_ from './FromIO';
import * as fromReader_ from './FromReader';
import { flow, identity, SK } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as I from './IO';
import * as reader from './Reader';
import * as readerT from './ReaderT';
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReader = /*#__PURE__*/ readerT.fromReader(I.Pointed);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = /*#__PURE__*/ reader.of;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = reader.local;
/**
 * Effectfully accesses the environment.
 *
 * @category combinators
 * @since 3.0.0
 */
export const asksReaderIO = reader.asksReader;
/**
 * @category Functor
 * @since 3.0.0
 */
export const map = /*#__PURE__*/ readerT.map(I.Functor);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/ readerT.ap(I.Apply);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = /*#__PURE__*/ readerT.of(I.Pointed);
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Monad
 * @since 3.0.0
 */
export const flatMap = 
/*#__PURE__*/ readerT.flatMap(I.Monad);
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = 
/*#__PURE__*/ flatMap(identity);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export const tap = 
/*#__PURE__*/ flattenable.tap(Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
// -------------------------------------------------------------------------------------
// logging
// -------------------------------------------------------------------------------------
/**
 * @category logging
 * @since 3.0.0
 */
export const log = /*#__PURE__*/ fromIO_.log(FromIO);
/**
 * @category logging
 * @since 3.0.0
 */
export const logError = 
/*#__PURE__*/ fromIO_.logError(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = /*#__PURE__*/ fromIO_.fromIOK(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOK = 
/*#__PURE__*/ fromIO_.flatMapIOK(FromIO, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromReader = {
    fromReader
};
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export const ask = /*#__PURE__*/ fromReader_.ask(FromReader);
/**
 * Projects a value from the global context in a `ReaderIO`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const asks = /*#__PURE__*/ fromReader_.asks(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderK = /*#__PURE__*/ fromReader_.fromReaderK(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapReaderK = /*#__PURE__*/ fromReader_.flatMapReaderK(FromReader, Flattenable);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = /*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(reader.traverseReadonlyNonEmptyArrayWithIndex(f), reader.map(I.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (f) => {
    return traverseReadonlyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = 
/*#__PURE__*/ traverseReadonlyArray(identity);
