import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as compactable from './Compactable';
import * as either from './Either';
import * as eitherT from './EitherT';
import * as filterable from './Filterable';
import * as fromEither_ from './FromEither';
import * as fromIO_ from './FromIO';
import * as fromReader_ from './FromReader';
import * as fromTask_ from './FromTask';
import { flow, identity, SK } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as reader from './Reader';
import * as readerTask from './ReaderTask';
import * as taskEither from './TaskEither';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTaskEither = 
/*#__PURE__*/ reader.of;
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = /*#__PURE__*/ eitherT.left(readerTask.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = /*#__PURE__*/ eitherT.right(readerTask.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightTask = /*#__PURE__*/ flow(taskEither.rightTask, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftTask = /*#__PURE__*/ flow(taskEither.leftTask, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightReader = (ma) => flow(ma, taskEither.right);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftReader = (me) => flow(me, taskEither.left);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightReaderTask = 
/*#__PURE__*/ eitherT.rightF(readerTask.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftReaderTask = 
/*#__PURE__*/ eitherT.leftF(readerTask.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightIO = /*#__PURE__*/ flow(taskEither.rightIO, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftIO = /*#__PURE__*/ flow(taskEither.leftIO, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const asksReaderTaskEither = reader.asksReader;
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightReaderIO = /*#__PURE__*/ (ma) => flow(ma, taskEither.rightIO);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftReaderIO = /*#__PURE__*/ (me) => flow(me, taskEither.leftIO);
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = readerTask.of;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReader = rightReader;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = rightIO;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTask = rightTask;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = /*#__PURE__*/ flow(taskEither.fromIOEither, fromTaskEither);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromReaderEither = (ma) => flow(ma, taskEither.fromEither);
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = /*#__PURE__*/ eitherT.match(readerTask.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchWithEffect = /*#__PURE__*/ eitherT.matchWithEffect(readerTask.Monad);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElse = 
/*#__PURE__*/ eitherT.getOrElse(readerTask.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseE = /*#__PURE__*/ eitherT.getOrElseE(readerTask.Monad);
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const toUnion = /*#__PURE__*/ eitherT.toUnion(readerTask.Functor);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = reader.local;
/**
 * @category combinators
 * @since 3.0.0
 */
export const orElse = /*#__PURE__*/ eitherT.orElse(readerTask.Monad);
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export const tapError = 
/*#__PURE__*/ eitherT.tapLeft(readerTask.Monad);
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = /*#__PURE__*/ eitherT.swap(readerTask.Functor);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOEitherK = (f) => (...a) => fromIOEither(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOEitherK = (f) => flatMap(fromIOEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskEitherK = (f) => (...a) => fromTaskEither(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapTaskEitherK = (f) => flatMap(fromTaskEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderEitherK = (f) => flow(f, fromReaderEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapReaderEitherK = (f) => flatMap(fromReaderEitherK(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderIOK = (f) => (...a) => rightReaderIO(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapReaderIOK = (f) => flatMap(fromReaderIOK(f));
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/ eitherT.map(readerTask.Functor);
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapBoth = /*#__PURE__*/ eitherT.mapBoth(readerTask.Functor);
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapError = 
/*#__PURE__*/ eitherT.mapLeft(readerTask.Functor);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/ eitherT.ap(readerTask.ApplyPar);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = /*#__PURE__*/ eitherT.flatMap(readerTask.Monad);
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ flatMap(identity);
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category SemigroupK
 * @since 3.0.0
 */
export const combineK = 
/*#__PURE__*/ eitherT.combineK(readerTask.Monad);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * The default [`ApplicativePar`](#applicativepar) instance returns the first error, if you want to
 * get all errors you need to provide an way to combine them via a `Semigroup`.
 *
 * See [`getValidatedApplicative`](./Either.ts.html#getvalidatedapplicative).
 *
 * @category instances
 * @since 3.0.0
 */
export const getValidatedApplicative = (A, S) => ({
    map,
    ap: apply.getApComposition(reader.Apply, taskEither.getValidatedApplicative(A, S)),
    of
});
/**
 * The default [`SemigroupK`](#semigroupk) instance returns the last error, if you want to
 * get all errors you need to provide an way to combine them via a `Semigroup`.
 *
 * @category instances
 * @since 3.0.0
 */
export const getValidatedSemigroupK = (S) => {
    return {
        combineK: eitherT.getValidatedCombineK(readerTask.Monad, S)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getCompactable = (M) => {
    const C = either.getCompactable(M);
    const F = { map: either.map };
    return {
        compact: compactable.getCompactComposition(readerTask.Functor, C),
        separate: compactable.getSeparateComposition(readerTask.Functor, C, F)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFilterable = (M) => {
    const F = either.getFilterable(M);
    return {
        filterMap: filterable.getFilterMapComposition(readerTask.Functor, F),
        partitionMap: filterable.getPartitionMapComposition(readerTask.Functor, F)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplyPar = {
    map,
    ap
};
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(ApplyPar);
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(ApplyPar);
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativePar = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
const apSeq = /*#__PURE__*/ flattenable.ap(Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplySeq = {
    map,
    ap: apSeq
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativeSeq = {
    map,
    ap: apSeq,
    of
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export const tap = 
/*#__PURE__*/ flattenable.tap(Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
// -------------------------------------------------------------------------------------
// logging
// -------------------------------------------------------------------------------------
/**
 * @category logging
 * @since 3.0.0
 */
export const log = 
/*#__PURE__*/ fromIO_.log(FromIO);
/**
 * @category logging
 * @since 3.0.0
 */
export const logError = 
/*#__PURE__*/ fromIO_.logError(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = /*#__PURE__*/ fromIO_.fromIOK(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOK = /*#__PURE__*/ fromIO_.flatMapIOK(FromIO, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromTask = {
    fromIO,
    fromTask
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskK = /*#__PURE__*/ fromTask_.fromTaskK(FromTask);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapTaskK = /*#__PURE__*/ fromTask_.flatMapTaskK(FromTask, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromReader = {
    fromReader
};
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export const ask = /*#__PURE__*/ fromReader_.ask(FromReader);
/**
 * Projects a value from the global context in a `ReaderEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const asks = /*#__PURE__*/ fromReader_.asks(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderK = /*#__PURE__*/ fromReader_.fromReaderK(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapReaderK = 
/*#__PURE__*/ fromReader_.flatMapReaderK(FromReader, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderTaskK = (f) => (...a) => rightReaderTask(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapReaderTaskK = (f) => flatMap(fromReaderTaskK(f));
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/ fromEither_.fromOption(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = /*#__PURE__*/ fromEither_.fromOptionK(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapOptionK = /*#__PURE__*/ fromEither_.flatMapOptionK(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapEitherK = 
/*#__PURE__*/ fromEither_.flatMapEitherK(FromEither, Flattenable);
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = /*#__PURE__*/ fromEither_.fromPredicate(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filter = /*#__PURE__*/ fromEither_.filter(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterMap = /*#__PURE__*/ fromEither_.filterMap(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partition = /*#__PURE__*/ fromEither_.partition(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partitionMap = 
/*#__PURE__*/ fromEither_.partitionMap(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = /*#__PURE__*/ fromEither_.fromEitherK(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = 
/*#__PURE__*/ fromEither_.fromNullable(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = /*#__PURE__*/ fromEither_.fromNullableK(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const flatMapNullableK = 
/*#__PURE__*/ fromEither_.flatMapNullableK(FromEither, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    mapBoth
};
/**
 * @category instances
 * @since 3.0.0
 */
export const SemigroupK = {
    combineK
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Make sure that a resource is cleaned up in the event of an exception (\*). The release action is called regardless of
 * whether the body action throws (\*) or returns.
 *
 * (\*) i.e. returns a `Left`
 *
 * @since 3.0.0
 */
export const bracket = /*#__PURE__*/ eitherT.bracket(readerTask.Monad);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(ApplyPar);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = 
/*#__PURE__*/ functor.tupled(Functor);
/**
 * @since 3.0.0
 */
export const apT = /*#__PURE__*/ apply.apT(ApplyPar);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
// --- Par ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplyPar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(reader.traverseReadonlyNonEmptyArrayWithIndex(f), reader.map(taskEither.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplyPar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (f) => {
    return traverseReadonlyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = /*#__PURE__*/ traverseReadonlyArray(identity);
// --- Seq ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplySeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndexSeq = (f) => flow(reader.traverseReadonlyNonEmptyArrayWithIndex(f), reader.map(taskEither.traverseReadonlyNonEmptyArrayWithIndexSeq(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndexSeq = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndexSeq(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplySeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArraySeq = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndexSeq((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArraySeq = (f) => {
    return traverseReadonlyArrayWithIndexSeq((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArraySeq = /*#__PURE__*/ traverseReadonlyArraySeq(identity);
