import * as fromWriter_ from './FromWriter';
import { flow, identity } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as reader from './Reader';
import * as readerTask from './ReaderTask';
import * as readonlyNonEmptyArray from './ReadonlyNonEmptyArray';
import * as task from './Task';
import * as writerT from './WriterT';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromReader = (w) => (fa) => fromReaderTask(w)(readerTask.fromReader(fa));
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromReaderTask = 
/*#__PURE__*/ writerT.fromF(readerTask.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromTaskWriter = /*#__PURE__*/ reader.of;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromIO = /*#__PURE__*/ writerT.fromIO(readerTask.Functor, readerTask.FromIO);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromTask = 
/*#__PURE__*/ writerT.fromTask(readerTask.Functor, readerTask.FromTask);
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export const tell = /*#__PURE__*/ writerT.tell(readerTask.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const asksReaderTaskWriter = reader.asksReader;
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromWriter = (fa) => readerTask.of(fa);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReaderWriter = (fa) => flow(fa, task.of);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const fst = /*#__PURE__*/ writerT.fst(readerTask.Functor);
/**
 * @since 3.0.0
 */
export const snd = /*#__PURE__*/ writerT.snd(readerTask.Functor);
/**
 * Alias of [`fst`](#fst).
 *
 * @since 3.0.0
 */
export const evaluate = fst;
/**
 * Alias of [`snd`](#snd).
 *
 * @since 3.0.0
 */
export const execute = snd;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = reader.local;
/**
 * @since 3.0.0
 */
export const swap = /*#__PURE__*/ writerT.swap(readerTask.Functor);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskWriterK = (f) => (...a) => fromTaskWriter(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderWriterK = (f) => flow(f, fromReaderWriter);
/**
 * @since 3.0.0
 */
export const listen = 
/*#__PURE__*/ writerT.listen(readerTask.Functor);
/**
 * @since 3.0.0
 */
export const pass = 
/*#__PURE__*/ writerT.pass(readerTask.Functor);
/**
 * @since 3.0.0
 */
export const listens = /*#__PURE__*/ writerT.listens(readerTask.Functor);
/**
 * @since 3.0.0
 */
export const censor = 
/*#__PURE__*/ writerT.censor(readerTask.Functor);
// -------------------------------------------------------------------------------------
// type class operations
// -------------------------------------------------------------------------------------
/**
 * @category type class operations
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/ writerT.map(readerTask.Functor);
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const mapError = 
/*#__PURE__*/ writerT.mapLeft(readerTask.Functor);
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapBoth = /*#__PURE__*/ writerT.mapBoth(readerTask.Functor);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    mapBoth
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const getPointed = (M) => ({
    of: writerT.of(readerTask.Pointed, M)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (A, S) => ({
    map,
    ap: writerT.ap(A, S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (A, M) => {
    const { ap } = getApply(A, M);
    const P = getPointed(M);
    return {
        map,
        ap,
        of: P.of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFlattenable = (S) => {
    return {
        map,
        flatMap: writerT.flatMap(readerTask.Flattenable, S)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (M) => {
    const P = getPointed(M);
    const C = getFlattenable(M);
    return {
        map,
        of: P.of,
        flatMap: C.flatMap
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromWriter = {
    fromWriter
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromWriterK = /*#__PURE__*/ fromWriter_.fromWriterK(FromWriter);
/**
 * @category instances
 * @since 3.0.0
 */
export const getFromReader = (M) => ({
    fromReader: fromReader(M.empty)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getFromIO = (M) => ({
    fromIO: fromIO(M.empty)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getFromTask = (M) => ({
    fromIO: fromIO(M.empty),
    fromTask: fromTask(M.empty)
});
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const tupled = 
/*#__PURE__*/ functor.tupled(Functor);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(A, M))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (A, S) => (f) => (as) => {
    // TODO
    return readonlyNonEmptyArray.traverseWithIndex(getApply(A, S))(f)(as);
};
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(A, M))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (A, M) => (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(A, M)(f);
    const P = getPointed(M);
    return (as) => (_.isNonEmpty(as) ? g(as) : P.of(_.emptyReadonlyArray));
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(A, M))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (A, S) => {
    const traverseReadonlyNonEmptyArrayWithIndexAM = traverseReadonlyNonEmptyArrayWithIndex(A, S);
    return (f) => {
        return traverseReadonlyNonEmptyArrayWithIndexAM((_, a) => f(a));
    };
};
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(A, M))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (A, M) => {
    const traverseReadonlyArrayWithIndexAM = traverseReadonlyArrayWithIndex(A, M);
    return (f) => {
        return traverseReadonlyArrayWithIndexAM((_, a) => f(a));
    };
};
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(A, M))`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = (A, M) => traverseReadonlyArray(A, M)(identity);
