import * as apply from './Apply';
import * as bifunctor from './Bifunctor';
import * as flattenable from './Flattenable';
import * as either from './Either';
import * as fromEither_ from './FromEither';
import * as fromIO_ from './FromIO';
import * as fromReader_ from './FromReader';
import * as fromState_ from './FromState';
import * as fromTask_ from './FromTask';
import { flow, identity, pipe } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as reader from './Reader';
import * as readerTaskEither from './ReaderTaskEither';
import * as stateT from './StateT';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = (e) => () => readerTaskEither.left(e);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = /*#__PURE__*/ stateT.of(readerTaskEither.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightTask = (ma) => fromReaderTaskEither(readerTaskEither.rightTask(ma));
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftTask = (me) => fromReaderTaskEither(readerTaskEither.leftTask(me));
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightReader = (ma) => fromReaderTaskEither(readerTaskEither.rightReader(ma));
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftReader = (me) => fromReaderTaskEither(readerTaskEither.leftReader(me));
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightIO = (ma) => fromReaderTaskEither(readerTaskEither.rightIO(ma));
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftIO = (me) => fromReaderTaskEither(readerTaskEither.leftIO(me));
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightState = 
/*#__PURE__*/ stateT.fromState(readerTaskEither.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftState = (me) => (s) => readerTaskEither.left(me(s)[1]);
/**
 * @category constructors
 * @since 3.0.0
 */
export const asksStateReaderTaskEither = (f) => (s) => (r) => f(r)(s)(r);
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = 
/*#__PURE__*/ either.match((e) => left(e), (a) => right(a));
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReader = rightReader;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = rightIO;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTask = rightTask;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromState = 
/*#__PURE__*/ stateT.fromState(readerTaskEither.Pointed);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTaskEither = (ma) => fromReaderTaskEither(readerTaskEither.fromTaskEither(ma));
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = (ma) => fromReaderTaskEither(readerTaskEither.fromIOEither(ma));
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromReaderEither = (ma) => fromReaderTaskEither(readerTaskEither.fromReaderEither(ma));
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromReaderTaskEither = 
/*#__PURE__*/ stateT.fromF(readerTaskEither.Functor);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = (f) => (ma) => flow(ma, reader.local(f));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOEitherK = (f) => (...a) => fromIOEither(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOEitherK = (f) => (ma) => pipe(ma, flatMap(fromIOEitherK(f)));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskEitherK = (f) => (...a) => fromTaskEither(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapTaskEitherK = (f) => (ma) => pipe(ma, flatMap(fromTaskEitherK(f)));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderTaskEitherK = (f) => (...a) => fromReaderTaskEither(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapReaderTaskEitherK = (f) => (ma) => pipe(ma, flatMap(fromReaderTaskEitherK(f)));
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = /*#__PURE__*/ stateT.map(readerTaskEither.Functor);
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapBoth = (f, g) => (fea) => (s) => pipe(fea(s), readerTaskEither.mapBoth(f, ([s, a]) => [s, g(a)]));
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapError = 
/*#__PURE__*/ bifunctor.getDefaultMapLeft(mapBoth);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/ stateT.ap(readerTaskEither.Monad);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * @since 3.0.0
 */
export const unit = () => of(undefined);
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = 
/*#__PURE__*/ stateT.flatMap(readerTaskEither.Monad);
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ flatMap(identity);
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category SemigroupK
 * @since 3.0.0
 */
export const combineK = (second) => (self) => (r) => pipe(self(r), readerTaskEither.combineK(() => second()(r)));
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(Apply);
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    mapBoth
};
/**
 * @category instances
 * @since 3.0.0
 */
export const SemigroupK = {
    combineK
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromState = {
    fromState
};
/**
 * Get the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export const get = /*#__PURE__*/ fromState_.get(FromState);
/**
 * Set the state
 *
 * @category constructors
 * @since 3.0.0
 */
export const put = /*#__PURE__*/ fromState_.put(FromState);
/**
 * Modify the state by applying a function to the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export const modify = 
/*#__PURE__*/ fromState_.modify(FromState);
/**
 * Get a value which depends on the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export const gets = 
/*#__PURE__*/ fromState_.gets(FromState);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromStateK = /*#__PURE__*/ fromState_.fromStateK(FromState);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapStateK = 
/*#__PURE__*/ fromState_.flatMapStateK(FromState, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export const tap = 
/*#__PURE__*/ flattenable.tap(Flattenable);
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export const tapError = (onError) => (self) => (s) => {
    return pipe(self(s), readerTaskEither.tapError((e1) => onError(e1)(s)));
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
// -------------------------------------------------------------------------------------
// logging
// -------------------------------------------------------------------------------------
/**
 * @category logging
 * @since 3.0.0
 */
export const log = 
/*#__PURE__*/ fromIO_.log(FromIO);
/**
 * @category logging
 * @since 3.0.0
 */
export const logError = 
/*#__PURE__*/ fromIO_.logError(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = /*#__PURE__*/ fromIO_.fromIOK(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOK = 
/*#__PURE__*/ fromIO_.flatMapIOK(FromIO, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromTask = {
    fromIO,
    fromTask
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskK = /*#__PURE__*/ fromTask_.fromTaskK(FromTask);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapTaskK = 
/*#__PURE__*/ fromTask_.flatMapTaskK(FromTask, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromReader = {
    fromReader
};
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export const ask = /*#__PURE__*/ fromReader_.ask(FromReader);
/**
 * Projects a value from the global context in a `ReaderEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const asks = 
/*#__PURE__*/ fromReader_.asks(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromReaderK = /*#__PURE__*/ fromReader_.fromReaderK(FromReader);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapReaderK = 
/*#__PURE__*/ fromReader_.flatMapReaderK(FromReader, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/ fromEither_.fromOption(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = /*#__PURE__*/ fromEither_.fromOptionK(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapOptionK = 
/*#__PURE__*/ fromEither_.flatMapOptionK(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapEitherK = 
/*#__PURE__*/ fromEither_.flatMapEitherK(FromEither, Flattenable);
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = /*#__PURE__*/ fromEither_.fromPredicate(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filter = /*#__PURE__*/ fromEither_.filter(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterMap = 
/*#__PURE__*/ fromEither_.filterMap(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partition = /*#__PURE__*/ fromEither_.partition(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partitionMap = 
/*#__PURE__*/ fromEither_.partitionMap(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = /*#__PURE__*/ fromEither_.fromEitherK(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = 
/*#__PURE__*/ fromEither_.fromNullable(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = 
/*#__PURE__*/ fromEither_.fromNullableK(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const flatMapNullableK = 
/*#__PURE__*/ fromEither_.flatMapNullableK(FromEither, Flattenable);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Run a computation in the `StateReaderTaskEither` monad, discarding the final state
 *
 * @since 3.0.0
 */
export const evaluate = 
/*#__PURE__*/ stateT.evaluate(readerTaskEither.Functor);
/**
 * Run a computation in the `StateReaderTaskEither` monad discarding the result
 *
 * @since 3.0.0
 */
export const execute = 
/*#__PURE__*/ stateT.execute(readerTaskEither.Functor);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const tupled = /*#__PURE__*/ functor.tupled(Functor);
/**
 * @since 3.0.0
 */
export const apT = /*#__PURE__*/ apply.apT(Apply);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => (as) => (s) => (r) => () => _.tail(as).reduce((acc, a, i) => acc.then((esb) => _.isLeft(esb)
    ? acc
    : f(i + 1, a)(esb.right[0])(r)().then((eb) => {
        if (_.isLeft(eb)) {
            return eb;
        }
        const [s, b] = eb.right;
        esb.right[1].push(b);
        esb.right[0] = s;
        return esb;
    })), f(0, _.head(as))(s)(r)().then(either.map(([s, b]) => [s, [b]])));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : of(_.emptyReadonlyArray));
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (f) => {
    return traverseReadonlyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = /*#__PURE__*/ traverseReadonlyArray(identity);
