import { identity, pipe } from './function';
import * as functor from './Functor';
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (fa) => ({
    peek: (s) => f(fa.peek(s)),
    pos: fa.pos
});
/**
 * @category Extendable
 * @since 3.0.0
 */
export const extend = (f) => (wa) => ({
    peek: (s) => f({ peek: wa.peek, pos: s }),
    pos: wa.pos
});
/**
 * @category Extract
 * @since 3.0.0
 */
export const extract = (wa) => wa.peek(wa.pos);
/**
 * Derivable from `Extendable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const duplicate = /*#__PURE__*/ extend(identity);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map,
    extend,
    extract
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Reposition the focus at the specified position.
 *
 * @since 3.0.0
 */
export const seek = (s) => (wa) => ({ peek: wa.peek, pos: s });
/**
 * Reposition the focus at the specified position, which depends on the current position.
 *
 * @since 3.0.0
 */
export const seeks = (f) => (wa) => ({ peek: wa.peek, pos: f(wa.pos) });
/**
 * Extract a value from a position which depends on the current position.
 *
 * @since 3.0.0
 */
export const peeks = (f) => (wa) => wa.peek(f(wa.pos));
/**
 * Extract a collection of values from positions which depend on the current position.
 *
 * @since 3.0.0
 */
export function experiment(F) {
    return (f) => (wa) => pipe(f(wa.pos), F.map((s) => wa.peek(s)));
}
