import * as monoidK from './MonoidK';
import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as compactable from './Compactable';
import * as filterable from './Filterable';
import * as fromOption_ from './FromOption';
import * as fromEither_ from './FromEither';
import * as fromIO_ from './FromIO';
import * as formTask_ from './FromTask';
import { flow, identity, SK } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as option from './Option';
import * as optionT from './OptionT';
import * as task from './Task';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const some = /*#__PURE__*/ optionT.some(task.Pointed);
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = task.of;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = /*#__PURE__*/ optionT.fromEither(task.Pointed);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = (ma) => fromTask(task.fromIO(ma));
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTask = /*#__PURE__*/ optionT.fromF(task.Functor);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = /*#__PURE__*/ flow(task.fromIO, task.map(option.fromEither));
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTaskEither = /*#__PURE__*/ task.map(option.fromEither);
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = 
/*#__PURE__*/ optionT.match(task.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchWithEffect = /*#__PURE__*/ optionT.matchWithEffect(task.Monad);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElse = 
/*#__PURE__*/ optionT.getOrElse(task.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseE = 
/*#__PURE__*/ optionT.getOrElseE(task.Monad);
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * Transforms a `Promise` that may reject to a `Promise` that never rejects and returns an `Option` instead.
 *
 * Note: `f` should never `throw` errors, they are not caught.
 *
 * See also [`tryCatchK`](#tryCatchK).
 *
 * @category interop
 * @since 3.0.0
 */
export const tryCatch = (f) => async () => {
    try {
        return await f().then(_.some);
    }
    catch (reason) {
        return _.none;
    }
};
/**
 * Converts a function returning a `Promise` to one returning a `TaskOption`.
 *
 * @category interop
 * @since 3.0.0
 */
export const tryCatchK = (f) => (...a) => tryCatch(() => f(...a));
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskEitherK = (f) => flow(f, fromTaskEither);
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = /*#__PURE__*/ optionT.map(task.Functor);
/**
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/ optionT.ap(task.ApplyPar);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = some;
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
/**
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = 
/*#__PURE__*/ optionT.flatMap(task.Monad);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapTaskEitherK = 
/*#__PURE__*/ flow(fromTaskEitherK, flatMap);
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ flatMap(identity);
/**
 * @category SemigroupK
 * @since 3.0.0
 */
export const combineK = 
/*#__PURE__*/ optionT.combineK(task.Monad);
/**
 * @category MonoidK
 * @since 3.0.0
 */
export const emptyK = /*#__PURE__*/ optionT.emptyK(task.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const none = /*#__PURE__*/ emptyK();
/**
 * @category Compactable
 * @since 3.0.0
 */
export const compact = 
/*#__PURE__*/ compactable.getCompactComposition(task.Functor, option.Compactable);
/**
 * @category Compactable
 * @since 3.0.0
 */
export const separate = 
/*#__PURE__*/ compactable.getSeparateComposition(task.Functor, option.Compactable, option.Functor);
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filterMap = 
/*#__PURE__*/ filterable.getFilterMapComposition(task.Functor, option.Filterable);
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partitionMap = 
/*#__PURE__*/ filterable.getPartitionMapComposition(task.Functor, option.Filterable);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplyPar = {
    map,
    ap
};
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(ApplyPar);
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(ApplyPar);
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativePar = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
const apSeq = /*#__PURE__*/ flattenable.ap(Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplySeq = {
    map,
    ap: apSeq
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativeSeq = {
    map,
    ap: apSeq,
    of
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export const tap = 
/*#__PURE__*/ flattenable.tap(Flattenable);
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export const tapError = 
/*#__PURE__*/ optionT.tapNone(task.Monad);
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const SemigroupK = {
    combineK
};
/**
 * @category instances
 * @since 3.0.0
 */
export const MonoidK = {
    combineK,
    emptyK
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const guard = /*#__PURE__*/ monoidK.guard(MonoidK, Pointed);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = 
/*#__PURE__*/ fromIO_.fromIOK(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOK = 
/*#__PURE__*/ fromIO_.flatMapIOK(FromIO, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromOption = {
    fromOption
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/ fromOption_.fromPredicate(FromOption);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromRefinement = /*#__PURE__*/ fromOption_.fromRefinement(FromOption);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = /*#__PURE__*/ fromOption_.fromOptionK(FromOption);
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = /*#__PURE__*/ fromOption_.fromNullable(FromOption);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = /*#__PURE__*/ fromOption_.fromNullableK(FromOption);
/**
 * @category interop
 * @since 3.0.0
 */
export const flatMapNullableK = /*#__PURE__*/ fromOption_.flatMapNullableK(FromOption, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = /*#__PURE__*/ fromEither_.fromEitherK(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapEitherK = 
/*#__PURE__*/ fromEither_.flatMapEitherK(FromEither, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromTask = {
    fromIO,
    fromTask
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskK = /*#__PURE__*/ formTask_.fromTaskK(FromTask);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapTaskK = 
/*#__PURE__*/ formTask_.flatMapTaskK(FromTask, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const Compactable = {
    compact,
    separate
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Filterable = {
    filterMap,
    partitionMap
};
/**
 * @since 3.0.0
 */
export const filter = /*#__PURE__*/ filterable.filter(Filterable);
/**
 * @since 3.0.0
 */
export const partition = /*#__PURE__*/ filterable.partition(Filterable);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(ApplyPar);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = /*#__PURE__*/ functor.tupled(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/ apply.apT(ApplyPar);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
// --- Par ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplyPar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(task.traverseReadonlyNonEmptyArrayWithIndex(f), task.map(option.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplyPar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (f) => {
    return traverseReadonlyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = 
/*#__PURE__*/ traverseReadonlyArray(identity);
// --- Seq ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplySeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndexSeq = (f) => (as) => () => _.tail(as).reduce((acc, a, i) => acc.then((obs) => _.isNone(obs)
    ? acc
    : f(i + 1, a)().then((ob) => {
        if (_.isNone(ob)) {
            return ob;
        }
        obs.value.push(ob.value);
        return obs;
    })), f(0, _.head(as))().then(option.map(_.singleton)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndexSeq = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndexSeq(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplySeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArraySeq = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndexSeq((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArraySeq = (f) => {
    return traverseReadonlyArrayWithIndexSeq((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArraySeq = 
/*#__PURE__*/ traverseReadonlyArraySeq(identity);
