import * as fromEither_ from './FromEither';
import * as fromIO_ from './FromIO';
import * as fromTask_ from './FromTask';
import * as fromThese_ from './FromThese';
import { flow, identity, SK } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as task from './Task';
import * as these from './These';
import * as theseT from './TheseT';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = /*#__PURE__*/ theseT.left(task.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = /*#__PURE__*/ theseT.right(task.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const both = /*#__PURE__*/ theseT.both(task.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightTask = /*#__PURE__*/ theseT.rightF(task.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftTask = /*#__PURE__*/ theseT.leftF(task.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightIO = /*#__PURE__*/ flow(task.fromIO, rightTask);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftIO = /*#__PURE__*/ flow(task.fromIO, leftTask);
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = /*#__PURE__*/ task.fromIO;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromEither = task.of;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromThese = task.of;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromIO = rightIO;
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromTask = rightTask;
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = /*#__PURE__*/ theseT.match(task.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchWithEffect = /*#__PURE__*/ theseT.matchWithEffect(task.Monad);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = /*#__PURE__*/ theseT.swap(task.Functor);
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = /*#__PURE__*/ theseT.map(task.Functor);
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapBoth = 
/*#__PURE__*/ theseT.mapBoth(task.Functor);
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapError = 
/*#__PURE__*/ theseT.mapLeft(task.Functor);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (A, S) => ({
    map,
    ap: theseT.ap(A, S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (A, S) => {
    const AS = getApply(A, S);
    return {
        map,
        ap: AS.ap,
        of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFlattenable = (S) => ({
    map,
    flatMap: theseT.flatMap(task.Monad, S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (S) => {
    const C = getFlattenable(S);
    return {
        map,
        of,
        flatMap: C.flatMap
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    mapBoth
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/ fromEither_.fromOption(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = /*#__PURE__*/ fromEither_.fromOptionK(FromEither);
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = /*#__PURE__*/ fromEither_.fromPredicate(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = /*#__PURE__*/ fromEither_.fromEitherK(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = 
/*#__PURE__*/ fromEither_.fromNullable(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = /*#__PURE__*/ fromEither_.fromNullableK(FromEither);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromThese = {
    fromThese
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTheseK = /*#__PURE__*/ fromThese_.fromTheseK(FromThese);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
// -------------------------------------------------------------------------------------
// logging
// -------------------------------------------------------------------------------------
/**
 * @category logging
 * @since 3.0.0
 */
export const log = /*#__PURE__*/ fromIO_.log(FromIO);
/**
 * @category logging
 * @since 3.0.0
 */
export const logError = /*#__PURE__*/ fromIO_.logError(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = 
/*#__PURE__*/ fromIO_.fromIOK(FromIO);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromTask = {
    fromIO,
    fromTask
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskK = /*#__PURE__*/ fromTask_.fromTaskK(FromTask);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const toTuple2 = 
/*#__PURE__*/ theseT.toTuple2(task.Functor);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
// --- Par ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(T.ApplyPar, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (S) => {
    const g = these.traverseReadonlyNonEmptyArrayWithIndex(S);
    return (f) => flow(task.traverseReadonlyNonEmptyArrayWithIndex(f), task.map(g(SK)));
};
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (S) => (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(S)(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(T.ApplyPar, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (S) => {
    const traverseReadonlyNonEmptyArrayWithIndexS = traverseReadonlyNonEmptyArrayWithIndex(S);
    return (f) => {
        return traverseReadonlyNonEmptyArrayWithIndexS((_, a) => f(a));
    };
};
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (S) => {
    const traverseReadonlyArrayWithIndexS = traverseReadonlyArrayWithIndex(S);
    return (f) => {
        return traverseReadonlyArrayWithIndexS((_, a) => f(a));
    };
};
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = (S) => traverseReadonlyArray(S)(identity);
// --- Seq ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(T.ApplySeq, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndexSeq = (S) => (f) => (as) => () => _.tail(as).reduce((acc, a, i) => acc.then((ebs) => these.isLeft(ebs)
    ? acc
    : f(i + 1, a)().then((eb) => {
        if (these.isLeft(eb)) {
            return eb;
        }
        if (these.isBoth(eb)) {
            const right = ebs.right;
            right.push(eb.right);
            return these.isBoth(ebs)
                ? these.both(S.combine(eb.left)(ebs.left), right)
                : these.both(eb.left, right);
        }
        ebs.right.push(eb.right);
        return ebs;
    })), f(0, _.head(as))().then(these.map(_.singleton)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndexSeq = (S) => (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndexSeq(S)(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(T.ApplySeq, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArraySeq = (S) => {
    const traverseReadonlyNonEmptyArrayWithIndexS = traverseReadonlyNonEmptyArrayWithIndexSeq(S);
    return (f) => {
        return traverseReadonlyNonEmptyArrayWithIndexS((_, a) => f(a));
    };
};
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArraySeq = (S) => {
    const traverseReadonlyArrayWithIndexS = traverseReadonlyArrayWithIndexSeq(S);
    return (f) => {
        return traverseReadonlyArrayWithIndexS((_, a) => f(a));
    };
};
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArraySeq = (S) => traverseReadonlyArraySeq(S)(identity);
