import { getApComposition as ap_ } from './Apply';
import { flow, pipe } from './function';
import { getMapComposition as map_ } from './Functor';
import * as T from './These';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const right = (F) => (a) => F.of(T.right(a));
/**
 * @since 3.0.0
 */
export const left = (F) => (e) => F.of(T.left(e));
/**
 * @since 3.0.0
 */
export const both = (F) => (e, a) => F.of(T.both(e, a));
/**
 * @since 3.0.0
 */
export function rightF(F) {
    return F.map(T.right);
}
/**
 * @since 3.0.0
 */
export function leftF(F) {
    return F.map(T.left);
}
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function map(F) {
    return map_(F, T.Functor);
}
/**
 * @since 3.0.0
 */
export const ap = (F, S) => {
    return ap_(F, T.getApply(S));
};
/**
 * @since 3.0.0
 */
export const flatMap = (M, S) => {
    const _left = left(M);
    return (f) => (self) => {
        return pipe(self, M.flatMap(T.match(_left, f, (e1, a) => pipe(f(a), M.map(T.match((e2) => T.left(S.combine(e2)(e1)), (b) => T.both(e1, b), (e2, b) => T.both(S.combine(e2)(e1), b)))))));
    };
};
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const mapBoth = (F) => {
    return flow(T.mapBoth, F.map);
};
/**
 * @category type class operations
 * @since 3.0.0
 */
export const mapLeft = (F) => (f) => {
    return F.map(T.mapError(f));
};
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function match(F) {
    return flow(T.match, F.map);
}
/**
 * @since 3.0.0
 */
export const matchWithEffect = (M) => (onError, onSuccess, onBoth) => {
    return M.flatMap(T.match(onError, onSuccess, onBoth));
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function swap(F) {
    return F.map(T.swap);
}
/**
 * @since 3.0.0
 */
export function toTuple2(F) {
    return flow(T.toTuple2, F.map);
}
