import * as functor from './Functor';
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (self) => (w) => f(self(w));
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const getComonad = (M) => ({
    map,
    extend: (f) => (self) => (w1) => f((w2) => self(M.combine(w2)(w1))),
    extract: (self) => self(M.empty)
});
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Extracts a value at a relative position which depends on the current value.
 *
 * @since 3.0.0
 */
export const tracks = (M) => (f) => (fa) => fa(f(fa(M.empty)));
/**
 * Get the current position.
 *
 * @since 3.0.0
 */
export const listen = (fa) => (w) => [fa(w), w];
/**
 * Get a value which depends on the current position.
 *
 * @since 3.0.0
 */
export const listens = (f) => (pa) => (w) => [pa(w), f(w)];
/**
 * Apply a function to the current position.
 *
 * @since 3.0.0
 */
export const censor = (f) => (pa) => (w) => pa(f(w));
