import * as apply from './Apply';
import { pipe } from './function';
import * as functor from './Functor';
import * as writer from './Writer';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export function fromF(F) {
    return (w) => F.map((a) => [w, a]);
}
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromIO = (F, FT) => (w) => (fa) => {
    return pipe(FT.fromIO(fa), F.map((a) => [w, a]));
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromTask = (F, FT) => (w) => (fa) => {
    return pipe(FT.fromTask(fa), F.map((a) => [w, a]));
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const tell = (F) => (w) => {
    return F.of(writer.tell(w));
};
// -------------------------------------------------------------------------------------
// type class operations
// -------------------------------------------------------------------------------------
/**
 * @category type class operations
 * @since 3.0.0
 */
export function map(F) {
    return functor.getMapComposition(F, writer.Functor);
}
/**
 * @category type class operations
 * @since 3.0.0
 */
export function of(F, M) {
    return (a) => F.of([M.empty, a]);
}
/**
 * @category type class operations
 * @since 3.0.0
 */
export const ap = (F, S) => {
    return apply.getApComposition(F, writer.getApply(S));
};
/**
 * @category type class operations
 * @since 3.0.0
 */
export const flatMap = (M, S) => (f) => {
    return M.flatMap(([w1, a]) => pipe(f(a), M.map(([w2, b]) => [S.combine(w2)(w1), b])));
};
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const mapBoth = (F) => {
    return (f, g) => F.map(writer.mapBoth(f, g));
};
/**
 * @category type class operations
 * @since 3.0.0
 */
export const mapLeft = (F) => (f) => {
    return F.map(writer.mapLeft(f));
};
// TODO: concatK, emptyK, fromEither, fromReader, fromState, reduce, foldMap, reduceRight, traverse, contramap
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function fst(F) {
    return F.map(writer.fst);
}
/**
 * @since 3.0.0
 */
export function snd(F) {
    return F.map(writer.snd);
}
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export function swap(F) {
    return F.map(writer.swap);
}
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export function listen(F) {
    return F.map(writer.listen);
}
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export function pass(F) {
    return F.map(writer.pass);
}
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export function listens(F) {
    return (f) => F.map(writer.listens(f));
}
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export function censor(F) {
    return (f) => F.map(writer.censor(f));
}
