import type { Functor } from './Functor'
import type { TypeLambda, Kind } from './HKT'
import * as semigroup from './Semigroup'
import type { Semigroup } from './Semigroup'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Apply<λ extends TypeLambda> extends Functor<λ> {
  readonly ap: <S, R2, O2, E2, A>(
    fa: Kind<λ, S, R2, O2, E2, A>
  ) => <R1, O1, E1, B>(self: Kind<λ, S, R1, O1, E1, (a: A) => B>) => Kind<λ, S, R1 & R2, O1 | O2, E1 | E2, B>
}
/**
 * `ap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const getApComposition: <λ extends TypeLambda, μ extends TypeLambda>(
  Applyλ: Apply<λ>,
  Applyμ: Apply<μ>
) => <λS, λR2, λO2, λE2, μS, μR2, μO2, μE2, A>(
  fa: Kind<λ, λS, λR2, λO2, λE2, Kind<μ, μS, μR2, μO2, μE2, A>>
) => <λR1, λO1, λE1, μR1, μO1, μE1, B>(
  self: Kind<λ, λS, λR1, λO1, λE1, Kind<μ, μS, μR1, μO1, μE1, (a: A) => B>>
) => Kind<λ, λS, λR1 & λR2, λO2 | λO1, λE2 | λE1, Kind<μ, μS, μR1 & μR2, μO2 | μO1, μE2 | μE1, B>>
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipLeftPar: <λ extends TypeLambda>(
  Applyλ: Apply<λ>
) => <S, R2, O2, E2, B>(
  second: Kind<λ, S, R2, O2, E2, B>
) => <R1, O1, E1, A>(self: Kind<λ, S, R1, O1, E1, A>) => Kind<λ, S, R1 & R2, O2 | O1, E2 | E1, A>
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipRightPar: <λ extends TypeLambda>(
  Applyλ: Apply<λ>
) => <S, R2, O2, E2, B>(
  second: Kind<λ, S, R2, O2, E2, B>
) => <R1, O1, E1, A>(self: Kind<λ, S, R1, O1, E1, A>) => Kind<λ, S, R1 & R2, O2 | O1, E2 | E1, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const bindPar: <λ extends TypeLambda>(
  Applyλ: Apply<λ>
) => <N extends string, A, S, R2, O2, E2, B>(
  name: Exclude<N, keyof A>,
  fb: Kind<λ, S, R2, O2, E2, B>
) => <R1, O1, E1>(
  self: Kind<λ, S, R1, O1, E1, A>
) => Kind<λ, S, R1 & R2, O2 | O1, E2 | E1, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const apT: <λ extends TypeLambda>(
  Applyλ: Apply<λ>
) => <S, R2, O2, E2, B>(
  fb: Kind<λ, S, R2, O2, E2, B>
) => <R1, O1, E1, A extends readonly unknown[]>(
  fas: Kind<λ, S, R1, O1, E1, A>
) => Kind<λ, S, R1 & R2, O2 | O1, E2 | E1, readonly [...A, B]>
/**
 * Lift a semigroup into 'λ', the inner values are combined using the provided `Semigroup`.
 *
 * @since 3.0.0
 */
export declare const getApplySemigroup: <λ extends TypeLambda>(
  Applyλ: Apply<λ>
) => <A, S, R, O, E>(S: semigroup.Semigroup<A>) => semigroup.Semigroup<Kind<λ, S, R, O, E, A>>
