/**
 * `Compactable` represents data structures which can be _compacted_/_separated_.
 *
 * @since 3.0.0
 */
import type { Either } from './Either'
import * as functor from './Functor'
import type { TypeLambda, Kind, TypeClass } from './HKT'
import type { Option } from './Option'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Compactable<F extends TypeLambda> extends TypeClass<F> {
  readonly compact: <S, R, W, E, A>(foa: Kind<F, S, R, W, E, Option<A>>) => Kind<F, S, R, W, E, A>
  readonly separate: <S, R, W, E, A, B>(
    fe: Kind<F, S, R, W, E, Either<A, B>>
  ) => readonly [Kind<F, S, R, W, E, A>, Kind<F, S, R, W, E, B>]
}
/**
 * Return a default `compact` implementation from `Functor` and `separate`.
 *
 * @category defaults
 * @since 3.0.0
 */
export declare const getDefaultCompact: <F extends TypeLambda>(
  F: functor.Functor<F>
) => (
  separate: <S, R, W, E, A, B>(
    fe: Kind<F, S, R, W, E, Either<A, B>>
  ) => readonly [Kind<F, S, R, W, E, A>, Kind<F, S, R, W, E, B>]
) => <S_1, R_1, W_1, E_1, A_1>(foa: Kind<F, S_1, R_1, W_1, E_1, Option<A_1>>) => Kind<F, S_1, R_1, W_1, E_1, A_1>
/**
 * Return a default `separate` implementation from `Functor` and `compact`.
 *
 * @category defaults
 * @since 3.0.0
 */
export declare function getDefaultSeparate<F extends TypeLambda>(
  F: functor.Functor<F>
): (compact: Compactable<F>['compact']) => Compactable<F>['separate']
/**
 * `compact` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function getCompactComposition<F extends TypeLambda, G extends TypeLambda>(
  F: functor.Functor<F>,
  G: Compactable<G>
): <FS, FR, FW, FE, GS, GR, GW, GE, A>(
  fgoa: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, Option<A>>>
) => Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>
/**
 * `separate` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function getSeparateComposition<F extends TypeLambda, G extends TypeLambda>(
  F: functor.Functor<F>,
  C: Compactable<G>,
  G: functor.Functor<G>
): <FS, FR, FW, FE, GS, GR, GW, GE, A, B>(
  fge: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, Either<A, B>>>
) => readonly [Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>, Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, B>>]
