/**
 * @since 3.0.0
 */
import * as apply from './Apply'
import type { Apply } from './Apply'
import type { Flattenable } from './Flattenable'
import * as either from './Either'
import type { Either } from './Either'
import type { LazyArg } from './function'
import * as functor from './Functor'
import type { Functor } from './Functor'
import type { TypeLambda, Kind } from './HKT'
import type { Monad } from './Monad'
import type { Pointed } from './Pointed'
import type { Semigroup } from './Semigroup'
/**
 * @since 3.0.0
 */
export declare const right: <F extends TypeLambda>(
  F: Pointed<F>
) => <A, S>(a: A) => Kind<F, S, unknown, never, never, either.Either<never, A>>
/**
 * @since 3.0.0
 */
export declare const left: <F extends TypeLambda>(
  F: Pointed<F>
) => <E, S>(e: E) => Kind<F, S, unknown, never, never, either.Either<E, never>>
/**
 * @since 3.0.0
 */
export declare function rightF<F extends TypeLambda>(
  F: Functor<F>
): <S, R, W, E, A>(fa: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, Either<never, A>>
/**
 * @since 3.0.0
 */
export declare function leftF<F extends TypeLambda>(
  F: Functor<F>
): <S, R, W, E, L>(fl: Kind<F, S, R, W, E, L>) => Kind<F, S, R, W, E, Either<L, never>>
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @since 3.0.0
 */
export declare const map: <F extends TypeLambda>(
  F: functor.Functor<F>
) => <A, B>(
  f: (a: A) => B
) => <S, R, W, FE, E>(self: Kind<F, S, R, W, FE, either.Either<E, A>>) => Kind<F, S, R, W, FE, either.Either<E, B>>
/**
 * @since 3.0.0
 */
export declare const ap: <F extends TypeLambda>(
  F: apply.Apply<F>
) => <S, R2, W2, FE2, E2, A>(
  fa: Kind<F, S, R2, W2, FE2, either.Either<E2, A>>
) => <R1, W1, FE1, E1, B>(
  self: Kind<F, S, R1, W1, FE1, either.Either<E1, (a: A) => B>>
) => Kind<F, S, R1 & R2, W2 | W1, FE2 | FE1, either.Either<E2 | E1, B>>
/**
 * @since 3.0.0
 */
export declare const flatMap: <M extends TypeLambda>(
  M: Monad<M>
) => <A, S, R2, W2, ME2, E2, B>(
  f: (a: A) => Kind<M, S, R2, W2, ME2, either.Either<E2, B>>
) => <R1, W1, ME1, E1>(
  self: Kind<M, S, R1, W1, ME1, either.Either<E1, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, either.Either<E2 | E1, B>>
/**
 * @since 3.0.0
 */
export declare const combineK: <M extends TypeLambda>(
  M: Monad<M>
) => <S, R2, W2, ME2, E2, B>(
  second: LazyArg<Kind<M, S, R2, W2, ME2, either.Either<E2, B>>>
) => <R1, W1, ME1, E1, A>(
  first: Kind<M, S, R1, W1, ME1, either.Either<E1, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, either.Either<E2, B | A>>
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapBoth: <F extends TypeLambda>(
  F: functor.Functor<F>
) => <E, G, A, B>(
  f: (e: E) => G,
  g: (a: A) => B
) => <S, R, W, FE>(self: Kind<F, S, R, W, FE, either.Either<E, A>>) => Kind<F, S, R, W, FE, either.Either<G, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapLeft: <F extends TypeLambda>(
  F: functor.Functor<F>
) => <E, G>(
  f: (e: E) => G
) => <S, R, W, FE, A>(self: Kind<F, S, R, W, FE, either.Either<E, A>>) => Kind<F, S, R, W, FE, either.Either<G, A>>
/**
 * @since 3.0.0
 */
export declare const getValidatedCombineK: <M extends TypeLambda, E>(
  M: Monad<M>,
  S: Semigroup<E>
) => <S, R2, W2, ME2, B>(
  second: LazyArg<Kind<M, S, R2, W2, ME2, either.Either<E, B>>>
) => <R1, W1, ME1, A>(
  first: Kind<M, S, R1, W1, ME1, either.Either<E, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, either.Either<E, B | A>>
/**
 * @since 3.0.0
 */
export declare function match<F extends TypeLambda>(
  F: Functor<F>
): <E, B, A, C = B>(
  onError: (e: E) => B,
  onSuccess: (a: A) => C
) => <S, R, W, ME>(self: Kind<F, S, R, W, ME, Either<E, A>>) => Kind<F, S, R, W, ME, B | C>
/**
 * @since 3.0.0
 */
export declare const matchWithEffect: <M extends TypeLambda>(
  M: Flattenable<M>
) => <E, S, R2, W2, ME2, B, A, R3, W3, ME3, C = B>(
  onError: (e: E) => Kind<M, S, R2, W2, ME2, B>,
  onSuccess: (a: A) => Kind<M, S, R3, W3, ME3, C>
) => <R1, W1, ME1>(
  self: Kind<M, S, R1, W1, ME1, either.Either<E, A>>
) => Kind<M, S, R1 & R2 & R3, W2 | W3 | W1, ME2 | ME3 | ME1, B | C>
/**
 * @since 3.0.0
 */
export declare const getOrElse: <F extends TypeLambda>(
  F: functor.Functor<F>
) => <E, B>(
  onError: (e: E) => B
) => <S, R, W, ME, A>(self: Kind<F, S, R, W, ME, either.Either<E, A>>) => Kind<F, S, R, W, ME, B | A>
/**
 * @since 3.0.0
 */
export declare const getOrElseE: <M extends TypeLambda>(
  M: Monad<M>
) => <E, S, R2, W2, ME2, B>(
  onError: (e: E) => Kind<M, S, R2, W2, ME2, B>
) => <R1, W1, ME1, A>(
  self: Kind<M, S, R1, W1, ME1, either.Either<E, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, B | A>
/**
 * @since 3.0.0
 */
export declare const orElse: <M extends TypeLambda>(
  M: Monad<M>
) => <E1, S, R2, W2, ME2, E2, B>(
  onError: (e: E1) => Kind<M, S, R2, W2, ME2, either.Either<E2, B>>
) => <R1, W1, ME1, A>(
  self: Kind<M, S, R1, W1, ME1, either.Either<E1, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, either.Either<E2, B | A>>
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export declare const tapLeft: <M extends TypeLambda>(
  M: Monad<M>
) => <E1, S, R2, W2, ME2, E2, _>(
  onError: (e: E1) => Kind<M, S, R2, W2, ME2, either.Either<E2, _>>
) => <R1, W1, ME1, A>(
  self: Kind<M, S, R1, W1, ME1, either.Either<E1, A>>
) => Kind<M, S, R1 & R2, W2 | W1, ME2 | ME1, either.Either<E1 | E2, A>>
/**
 * @since 3.0.0
 */
export declare function swap<F extends TypeLambda>(
  F: Functor<F>
): <S, R, W, FE, E, A>(self: Kind<F, S, R, W, FE, Either<E, A>>) => Kind<F, S, R, W, FE, Either<A, E>>
/**
 * @since 3.0.0
 */
export declare function toUnion<F extends TypeLambda>(
  F: Functor<F>
): <S, R, W, FE, E, A>(self: Kind<F, S, R, W, FE, Either<E, A>>) => Kind<F, S, R, W, FE, E | A>
/**
 * @since 3.0.0
 */
export declare const bracket: <M extends TypeLambda>(
  M: Monad<M>
) => <S, R1, W1, ME1, E1, A, R2, W2, ME2, E2, B, R3, W3, ME3, E3>(
  acquire: Kind<M, S, R1, W1, ME1, either.Either<E1, A>>,
  use: (a: A) => Kind<M, S, R2, W2, ME2, either.Either<E2, B>>,
  release: (a: A, e: either.Either<E2, B>) => Kind<M, S, R3, W3, ME3, either.Either<E3, void>>
) => Kind<M, S, R1 & R2 & R3, W1 | W2 | W3, ME1 | ME2 | ME3, either.Either<E1 | E2 | E3, B>>
