/**
 * `Filterable` represents data structures which can be _partitioned_/_filtered_.
 *
 * @since 3.0.0
 */
import type { Either } from './Either'
import type { Functor } from './Functor'
import type { TypeLambda, Kind, TypeClass } from './HKT'
import type { Option } from './Option'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Filterable<F extends TypeLambda> extends TypeClass<F> {
  readonly partitionMap: <A, B, C>(
    f: (a: A) => Either<B, C>
  ) => <S, R, W, E>(self: Kind<F, S, R, W, E, A>) => readonly [Kind<F, S, R, W, E, B>, Kind<F, S, R, W, E, C>]
  readonly filterMap: <A, B>(
    f: (a: A) => Option<B>
  ) => <S, R, W, E>(self: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, B>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const filter: <F extends TypeLambda>(
  F: Filterable<F>
) => {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): <S, R, W, E>(
    self: Kind<F, S, R, W, E, C>
  ) => Kind<F, S, R, W, E, B>
  <B extends A, A = B>(predicate: Predicate<A>): <S, R, W, E>(self: Kind<F, S, R, W, E, B>) => Kind<F, S, R, W, E, B>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const partition: <F extends TypeLambda>(
  F: Filterable<F>
) => {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): <S, R, W, E>(
    self: Kind<F, S, R, W, E, C>
  ) => readonly [Kind<F, S, R, W, E, C>, Kind<F, S, R, W, E, B>]
  <B extends A, A = B>(predicate: Predicate<A>): <S, R, W, E>(
    self: Kind<F, S, R, W, E, B>
  ) => readonly [Kind<F, S, R, W, E, B>, Kind<F, S, R, W, E, B>]
}
/**
 * `filterMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const getFilterMapComposition: <F extends TypeLambda, G extends TypeLambda>(
  F: Functor<F>,
  G: Filterable<G>
) => <A, B>(
  f: (a: A) => Option<B>
) => <FS, FR, FW, FE, GS, GR, GW, GE>(
  self: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>
) => Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, B>>
/**
 * `partitionMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const getPartitionMapComposition: <F extends TypeLambda, G extends TypeLambda>(
  F: Functor<F>,
  G: Filterable<G>
) => <A, B, C>(
  f: (a: A) => Either<B, C>
) => <FS, FR, FW, FE, GS, GR, GW, GE>(
  self: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>
) => readonly [Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, B>>, Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, C>>]
