/**
 * `FilterableWithEffect` represents data structures which can be _partitioned_ with effects in some `Applicative` functor.
 *
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type { Compactable } from './Compactable'
import type { Either } from './Either'
import type { TypeLambda, Kind, TypeClass } from './HKT'
import type { Option } from './Option'
import type { Traversable } from './Traversable'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FilterableWithEffect<T extends TypeLambda> extends TypeClass<T> {
  readonly partitionMapWithEffect: <F extends TypeLambda>(
    F: Applicative<F>
  ) => <A, S, R, W, E, B, C>(
    f: (a: A) => Kind<F, S, R, W, E, Either<B, C>>
  ) => <TS, TR, TW, TE>(
    wa: Kind<T, TS, TR, TW, TE, A>
  ) => Kind<F, S, R, W, E, readonly [Kind<T, TS, TR, TW, TE, B>, Kind<T, TS, TR, TW, TE, C>]>
  readonly filterMapWithEffect: <F extends TypeLambda>(
    F: Applicative<F>
  ) => <A, S, R, W, E, B>(
    f: (a: A) => Kind<F, S, R, W, E, Option<B>>
  ) => <TS, TR, TW, TE>(ta: Kind<T, TS, TR, TW, TE, A>) => Kind<F, S, R, W, E, Kind<T, TS, TR, TW, TE, B>>
}
/**
 * Return a `partitionMapWithEffect` implementation from `Traversable` and `Compactable`.
 *
 * @category defaults
 * @since 3.0.0
 */
export declare function getDefaultPartitionMapWithEffect<T extends TypeLambda>(
  T: Traversable<T>,
  C: Compactable<T>
): FilterableWithEffect<T>['partitionMapWithEffect']
/**
 * Return a `filterMapWithEffect` implementation from `Traversable` and `Compactable`.
 *
 * @category defaults
 * @since 3.0.0
 */
export declare function getDefaultFilterMapWithEffect<T extends TypeLambda>(
  T: Traversable<T>,
  C: Compactable<T>
): FilterableWithEffect<T>['filterMapWithEffect']
/**
 * Filter values inside a `F` context.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const filterWithEffect: <μ extends TypeLambda>(
  FilterableWithEffectμ: FilterableWithEffect<μ>
) => <λ extends TypeLambda>(
  Applicativeλ: Applicative<λ>
) => <B extends A, S, R, O, E, A = B>(
  predicateK: (a: A) => Kind<λ, S, R, O, E, boolean>
) => <μS, μR, μW, μE>(self: Kind<μ, μS, μR, μW, μE, B>) => Kind<λ, S, R, O, E, Kind<μ, μS, μR, μW, μE, B>>
/**
 * Partition values inside a `F` context.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const partitionWithEffect: <μ extends TypeLambda>(
  FilterableWithEffectμ: FilterableWithEffect<μ>
) => <λ extends TypeLambda>(
  Applicativeλ: Applicative<λ>
) => <B extends A, S, R, O, E, A = B>(
  predicateK: (a: A) => Kind<λ, S, R, O, E, boolean>
) => <μS, μR, μW, μE>(
  self: Kind<μ, μS, μR, μW, μE, B>
) => Kind<λ, S, R, O, E, readonly [Kind<μ, μS, μR, μW, μE, B>, Kind<μ, μS, μR, μW, μE, B>]>
