/**
 * @since 3.0.0
 */
import type { Apply } from './Apply'
import type { Functor } from './Functor'
import type { TypeLambda, Kind } from './HKT'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Flattenable<M extends TypeLambda> extends Functor<M> {
  readonly flatMap: <A, S, R2, W2, E2, B>(
    f: (a: A) => Kind<M, S, R2, W2, E2, B>
  ) => <R1, W1, E1>(self: Kind<M, S, R1, W1, E1, A>) => Kind<M, S, R1 & R2, W1 | W2, E1 | E2, B>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const ap: <F extends TypeLambda>(
  M: Flattenable<F>
) => <S, R2, O2, E2, A>(
  fa: Kind<F, S, R2, O2, E2, A>
) => <R1, O1, E1, B>(self: Kind<F, S, R1, O1, E1, (a: A) => B>) => Kind<F, S, R1 & R2, O2 | O1, E2 | E1, B>
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const tap: <M extends TypeLambda>(
  M: Flattenable<M>
) => <A, S, R2, W2, E2, _>(
  f: (a: A) => Kind<M, S, R2, W2, E2, _>
) => <R1, W1, E1>(self: Kind<M, S, R1, W1, E1, A>) => Kind<M, S, R1 & R2, W2 | W1, E2 | E1, A>
/**
 * @since 3.0.0
 */
export declare const bind: <M extends TypeLambda>(
  M: Flattenable<M>
) => <N extends string, A, S, R2, W2, E2, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => Kind<M, S, R2, W2, E2, B>
) => <R1, W1, E1>(
  self: Kind<M, S, R1, W1, E1, A>
) => Kind<M, S, R1 & R2, W2 | W1, E2 | E1, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
