import type { TypeLambda, Kind, TypeClass } from './HKT'
import type { Monoid } from './Monoid'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FoldableWithIndex<F extends TypeLambda, I> extends TypeClass<F> {
  readonly reduceWithIndex: <B, A>(b: B, f: (i: I, b: B, a: A) => B) => <S, R, W, E>(self: Kind<F, S, R, W, E, A>) => B
  readonly foldMapWithIndex: <M>(
    M: Monoid<M>
  ) => <A>(f: (i: I, a: A) => M) => <S, R, W, E>(self: Kind<F, S, R, W, E, A>) => M
  readonly reduceRightWithIndex: <B, A>(
    b: B,
    f: (i: I, a: A, b: B) => B
  ) => <S, R, W, E>(self: Kind<F, S, R, W, E, A>) => B
}
/**
 * `reduceWithIndex` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const getReduceWithIndexComposition: <F extends TypeLambda, I, G extends TypeLambda, J>(
  F: FoldableWithIndex<F, I>,
  G: FoldableWithIndex<G, J>
) => <B, A>(
  b: B,
  f: (i: readonly [I, J], b: B, a: A) => B
) => <FS, FR, FW, FE, GS, GR, GW, GE>(fga: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>) => B
/**
 * `foldMapWithIndex` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const getFoldMapWithIndexComposition: <F extends TypeLambda, I, G extends TypeLambda, J>(
  F: FoldableWithIndex<F, I>,
  G: FoldableWithIndex<G, J>
) => <M>(
  M: Monoid<M>
) => <A>(
  f: (i: readonly [I, J], a: A) => M
) => <FS, FR, FW, FE, GS, GR, GW, GE>(fga: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>) => M
/**
 * `reduceRightWithIndex` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const getReduceRightWithIndexComposition: <F extends TypeLambda, I, G extends TypeLambda, J>(
  F: FoldableWithIndex<F, I>,
  G: FoldableWithIndex<G, J>
) => <B, A>(
  b: B,
  f: (i: readonly [I, J], a: A, b: B) => B
) => <FS, FR, FW, FE, GS, GR, GW, GE>(fga: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>) => B
