/**
 * Lift a computation from the `IO` monad.
 *
 * @since 3.0.0
 */
import type { Flattenable } from './Flattenable'
import type { TypeLambda, Kind, TypeClass } from './HKT'
import type { IO } from './IO'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FromIO<F extends TypeLambda> extends TypeClass<F> {
  readonly fromIO: <A, S>(fa: IO<A>) => Kind<F, S, unknown, never, never, A>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <F extends TypeLambda>(
  F: FromIO<F>
) => <A extends readonly unknown[], B>(f: (...a: A) => IO<B>) => <S>(...a: A) => Kind<F, S, unknown, never, never, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapIOK: <M extends TypeLambda>(
  F: FromIO<M>,
  M: Flattenable<M>
) => <A, B>(f: (a: A) => IO<B>) => <S, R, W, E>(self: Kind<M, S, R, W, E, A>) => Kind<M, S, R, W, E, B>
/**
 * @category logging
 * @since 3.0.0
 */
export declare const log: <M extends TypeLambda>(
  F: FromIO<M>
) => <S>(...x: ReadonlyArray<unknown>) => Kind<M, S, unknown, never, never, void>
/**
 * @category logging
 * @since 3.0.0
 */
export declare const logError: <M extends TypeLambda>(
  F: FromIO<M>
) => <S>(...x: ReadonlyArray<unknown>) => Kind<M, S, unknown, never, never, void>
