/**
 * The `FromEither` type class represents those data types which support untyped errors.
 *
 * @since 3.0.0
 */
import type { Flattenable } from './Flattenable'
import type { TypeLambda, Kind, TypeClass } from './HKT'
import type { Option } from './Option'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FromOption<F extends TypeLambda> extends TypeClass<F> {
  readonly fromOption: <A, S>(fa: Option<A>) => Kind<F, S, unknown, never, never, A>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: <F extends TypeLambda>(
  F: FromOption<F>
) => <B extends A, A = B>(predicate: Predicate<A>) => <S>(b: B) => Kind<F, S, unknown, never, never, B>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromRefinement: <F extends TypeLambda>(
  F: FromOption<F>
) => <C extends A, B extends A, A = C>(
  refinement: Refinement<A, B>
) => <S>(c: C) => Kind<F, S, unknown, never, never, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <F extends TypeLambda>(
  F: FromOption<F>
) => <A extends readonly unknown[], B>(
  f: (...a: A) => Option<B>
) => <S>(...a: A) => Kind<F, S, unknown, never, never, B>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <F extends TypeLambda>(
  F: FromOption<F>
) => <A, S, R, W, E>(a: A) => Kind<F, S, R, W, E, NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <F extends TypeLambda>(
  F: FromOption<F>
) => <A extends readonly unknown[], B>(
  f: (...a: A) => B | null | undefined
) => <S, R, W, E>(...a: A) => Kind<F, S, R, W, E, NonNullable<B>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const flatMapNullableK: <F extends TypeLambda>(
  F: FromOption<F>,
  C: Flattenable<F>
) => <A, B>(
  f: (a: A) => B | null | undefined
) => <S, R, W, E>(self: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, NonNullable<B>>
