/**
 * Lift a computation from the `Reader` monad.
 *
 * @since 3.0.0
 */
import type { Flattenable } from './Flattenable'
import type { TypeLambda, Kind, TypeClass } from './HKT'
import type { Reader } from './Reader'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FromReader<F extends TypeLambda> extends TypeClass<F> {
  readonly fromReader: <R, A, S>(fa: Reader<R, A>) => Kind<F, S, R, never, never, A>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare function ask<F extends TypeLambda>(F: FromReader<F>): <S, R>() => Kind<F, S, R, never, never, R>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare function asks<F extends TypeLambda>(
  F: FromReader<F>
): <R, A, S>(f: (r: R) => A) => Kind<F, S, R, never, never, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromReaderK: <F extends TypeLambda>(
  F: FromReader<F>
) => <A extends readonly unknown[], R, B>(
  f: (...a: A) => Reader<R, B>
) => <S>(...a: A) => Kind<F, S, R, never, never, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapReaderK: <M extends TypeLambda>(
  F: FromReader<M>,
  M: Flattenable<M>
) => <A, R2, B>(
  f: (a: A) => Reader<R2, B>
) => <S, R1, W, E>(self: Kind<M, S, R1, W, E, A>) => Kind<M, S, R1 & R2, W, E, B>
