/**
 * Lift a computation from the `State` monad.
 *
 * @since 3.0.0
 */
import type { Flattenable } from './Flattenable'
import type { Endomorphism } from './Endomorphism'
import type { TypeLambda, Kind, TypeClass } from './HKT'
import * as state from './State'
import type { State } from './State'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FromState<F extends TypeLambda> extends TypeClass<F> {
  readonly fromState: <S, A>(fa: State<S, A>) => Kind<F, S, unknown, never, never, A>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare function get<F extends TypeLambda>(F: FromState<F>): <S>() => Kind<F, S, unknown, never, never, S>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare function put<F extends TypeLambda>(F: FromState<F>): <S>(s: S) => Kind<F, S, unknown, never, never, void>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const modify: <F extends TypeLambda>(
  F: FromState<F>
) => <S>(f: Endomorphism<S>) => Kind<F, S, unknown, never, never, void>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const gets: <F extends TypeLambda>(
  F: FromState<F>
) => <S, A>(f: (s: S) => A) => Kind<F, S, unknown, never, never, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromStateK: <F extends TypeLambda>(
  F: FromState<F>
) => <A extends readonly unknown[], S, B>(
  f: (...a: A) => state.State<S, B>
) => (...a: A) => Kind<F, S, unknown, never, never, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapStateK: <M extends TypeLambda>(
  F: FromState<M>,
  M: Flattenable<M>
) => <A, S, B>(f: (a: A) => state.State<S, B>) => <R, W, E>(self: Kind<M, S, R, W, E, A>) => Kind<M, S, R, W, E, B>
