import type { TypeLambda, Kind, TypeClass } from './HKT'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Functor<λ extends TypeLambda> extends TypeClass<λ> {
  readonly map: <A, B>(f: (a: A) => B) => <S, R, O, E>(self: Kind<λ, S, R, O, E, A>) => Kind<λ, S, R, O, E, B>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <λ extends TypeLambda>(
  Functorλ: Functor<λ>
) => <A>(a: A) => <S, R, O, E, B>(self: Kind<λ, S, R, O, E, (a: A) => B>) => Kind<λ, S, R, O, E, B>
/**
 * `map` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const getMapComposition: <λ extends TypeLambda, μ extends TypeLambda>(
  Functorλ: Functor<λ>,
  Functorμ: Functor<μ>
) => <A, B>(
  f: (a: A) => B
) => <λS, λR, λO, λE, μS, μR, μO, μE>(
  self: Kind<λ, λS, λR, λO, λE, Kind<μ, μS, μR, μO, μE, A>>
) => Kind<λ, λS, λR, λO, λE, Kind<μ, μS, μR, μO, μE, B>>
/**
 * @since 3.0.0
 */
export declare const bindTo: <λ extends TypeLambda>(
  Functorλ: Functor<λ>
) => <N extends string>(
  name: N
) => <S, R, O, E, A>(self: Kind<λ, S, R, O, E, A>) => Kind<λ, S, R, O, E, { readonly [K in N]: A }>
/**
 * @since 3.0.0
 */
export declare const tupled: <λ extends TypeLambda>(
  Functorλ: Functor<λ>
) => <S, R, O, E, A>(self: Kind<λ, S, R, O, E, A>) => Kind<λ, S, R, O, E, readonly [A]>
declare const let_: <λ extends TypeLambda>(
  Functorλ: Functor<λ>
) => <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <S, R, O, E>(
  self: Kind<λ, S, R, O, E, A>
) => Kind<λ, S, R, O, E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
