/**
 * ```ts
 * export type Identity<A> = A
 * ```
 *
 * @since 3.0.0
 */
import type * as semigroupK from './SemigroupK'
import * as apply from './Apply'
import type * as applicative from './Applicative'
import * as flattenable from './Flattenable'
import * as flattenableRec from './FlattenableRec'
import type * as comonad from './Comonad'
import type { Either } from './Either'
import type { Eq } from './Eq'
import type * as extendable from './Extendable'
import type * as foldable from './Foldable'
import type { LazyArg } from './function'
import * as functor from './Functor'
import type { TypeLambda, Kind } from './HKT'
import type * as monad from './Monad'
import type { Monoid } from './Monoid'
import type * as pointed from './Pointed'
import type { Show } from './Show'
import * as traversable from './Traversable'
/**
 * @category model
 * @since 3.0.0
 */
export declare type Identity<A> = A
/**
 * @category model
 * @since 3.0.0
 */
export interface Identityλ extends TypeLambda {
  readonly type: Identity<this['Out1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <A>(S: Show<A>) => Show<Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <A>(E: Eq<A>) => Eq<Identity<A>>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: Identity<A>) => Identity<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<Identityλ>
/**
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: Identity<(a: A) => B>) => B
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => Identity<A>
/**
 * @since 3.0.0
 */
export declare const unit: Identity<void>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: pointed.Pointed<Identityλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const ap: <A>(fa: Identity<A>) => <B>(fab: Identity<(a: A) => B>) => Identity<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: apply.Apply<Identityλ>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipLeftPar: <B>(second: Identity<B>) => <A>(self: Identity<A>) => Identity<A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipRightPar: <B>(second: Identity<B>) => <A>(self: Identity<A>) => Identity<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: applicative.Applicative<Identityλ>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMap: <A, B>(f: (a: A) => Identity<B>) => (ma: Identity<A>) => Identity<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Flattenable: flattenable.Flattenable<Identityλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: Identity<Identity<A>>) => Identity<A>
/**
 * @since 3.0.0
 */
export declare const flatMapRec: <A, B>(f: (a: A) => Identity<Either<A, B>>) => (a: A) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FlattenableRec: flattenableRec.FlattenableRec<Identityλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: monad.Monad<Identityλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const extend: <A, B>(f: (wa: Identity<A>) => B) => (wa: Identity<A>) => B
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const duplicate: <A>(ma: Identity<A>) => Identity<Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Extendable: extendable.Extendable<Identityλ>
/**
 * @since 3.0.0
 */
export declare const extract: <A>(wa: Identity<A>) => A
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: comonad.Comonad<Identityλ>
/**
 * @since 3.0.0
 */
export declare const reduce: <B, A>(b: B, f: (b: B, a: A) => B) => (fa: Identity<A>) => B
/**
 * @since 3.0.0
 */
export declare const foldMap: <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: Identity<A>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRight: <B, A>(b: B, f: (a: A, b: B) => B) => (fa: Identity<A>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: foldable.Foldable<Identityλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const traverse: <F extends TypeLambda>(
  F: applicative.Applicative<F>
) => <A, S, R, W, E, B>(f: (a: A) => Kind<F, S, R, W, E, B>) => (ta: Identity<A>) => Kind<F, S, R, W, E, Identity<B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: traversable.Traversable<Identityλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const sequence: <F extends TypeLambda>(
  F: applicative.Applicative<F>
) => <S, R, W, E, A>(fas: Identity<Kind<F, S, R, W, E, A>>) => Kind<F, S, R, W, E, Identity<A>>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const combineK: <B>(second: LazyArg<Identity<B>>) => <A>(self: Identity<A>) => Identity<A | B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupK: semigroupK.SemigroupK<Identityλ>
/**
 * @category do notation
 * @since 3.0.0
 */
export declare const Do: Identity<{}>
/**
 * @category do notation
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(fa: Identity<A>) => {
  readonly [K in N]: A
}
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (fa: A) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
export {
  /**
   * @category do notation
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @category do notation
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => Identity<B>
) => (ma: Identity<A>) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
/**
 * @category do notation
 * @since 3.0.0
 */
export declare const bindPar: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: Identity<B>
) => (fa: Identity<A>) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
/**
 * @category do notation
 * @since 3.0.0
 */
export declare const ApT: Identity<readonly []>
/**
 * @category do notation
 * @since 3.0.0
 */
export declare const tupled: <A>(fa: Identity<A>) => readonly [A]
/**
 * @category do notation
 * @since 3.0.0
 */
export declare const apT: <B>(fb: B) => <A extends ReadonlyArray<unknown>>(fas: A) => readonly [...A, B]
