/**
 * @since 3.0.0
 */
import type * as semigroupK from './SemigroupK'
import type * as applicative from './Applicative'
import * as apply from './Apply'
import type * as bifunctor from './Bifunctor'
import * as flattenable from './Flattenable'
import * as compactable from './Compactable'
import * as either from './Either'
import type * as filterable from './Filterable'
import * as fromEither_ from './FromEither'
import * as fromReader_ from './FromReader'
import type { LazyArg } from './function'
import * as functor from './Functor'
import type { TypeLambda } from './HKT'
import type * as monad from './Monad'
import type { Monoid } from './Monoid'
import type { Option } from './Option'
import type * as pointed from './Pointed'
import type { Predicate } from './Predicate'
import * as reader from './Reader'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Refinement } from './Refinement'
import type { Semigroup } from './Semigroup'
import Either = either.Either
import Reader = reader.Reader
/**
 * @category model
 * @since 3.0.0
 */
export interface ReaderEither<R, E, A> extends Reader<R, Either<E, A>> {}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ReaderEitherλ extends TypeLambda {
  readonly type: ReaderEither<this['In1'], this['Out2'], this['Out1']>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const left: <E>(e: E) => ReaderEither<unknown, E, never>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const right: <A>(a: A) => ReaderEither<unknown, never, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const rightReader: <R, A>(ma: Reader<R, A>) => ReaderEither<R, never, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const leftReader: <R, E>(me: Reader<R, E>) => ReaderEither<R, E, never>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const asksReaderEither: <R1, R2, E, A>(
  f: (r1: R1) => ReaderEither<R2, E, A>
) => ReaderEither<R1 & R2, E, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromEither: <E, A>(fa: Either<E, A>) => ReaderEither<unknown, E, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromReader: <R, A>(fa: Reader<R, A>) => ReaderEither<R, never, A>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <E, B, A, C = B>(
  onError: (e: E) => B,
  onSuccess: (a: A) => C
) => <R>(ma: Reader<R, either.Either<E, A>>) => Reader<R, B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const matchWithEffect: <E, R2, B, A, R3, C = B>(
  onError: (e: E) => Reader<R2, B>,
  onSuccess: (a: A) => Reader<R3, C>
) => <R1>(ma: Reader<R1, either.Either<E, A>>) => Reader<R1 & R2 & R3, B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElse: <E, B>(onError: (e: E) => B) => <R, A>(ma: ReaderEither<R, E, A>) => Reader<R, A | B>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElseE: <E, R2, B>(
  onError: (e: E) => Reader<R2, B>
) => <R1, A>(ma: ReaderEither<R1, E, A>) => Reader<R1 & R2, A | B>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const toUnion: <R, E, A>(fa: ReaderEither<R, E, A>) => Reader<R, E | A>
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const local: <R2, R1>(f: (r2: R2) => R1) => <E, A>(ma: ReaderEither<R1, E, A>) => ReaderEither<R2, E, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const orElse: <E1, R1, E2, B>(
  onError: (e: E1) => ReaderEither<R1, E2, B>
) => <R2, A>(ma: ReaderEither<R2, E1, A>) => ReaderEither<R1 & R2, E2, A | B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const swap: <R, E, A>(ma: ReaderEither<R, E, A>) => ReaderEither<R, A, E>
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <R, E>(fa: ReaderEither<R, E, A>) => ReaderEither<R, E, B>
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapBoth: <E, G, A, B>(
  f: (e: E) => G,
  g: (a: A) => B
) => <R>(self: ReaderEither<R, E, A>) => ReaderEither<R, G, B>
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapError: <E, G>(f: (e: E) => G) => <R, A>(self: ReaderEither<R, E, A>) => ReaderEither<R, G, A>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <R2, E2, A>(
  fa: ReaderEither<R2, E2, A>
) => <R1, E1, B>(fab: ReaderEither<R1, E1, (a: A) => B>) => ReaderEither<R1 & R2, E1 | E2, B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => ReaderEither<unknown, never, A>
/**
 * @since 3.0.0
 */
export declare const unit: ReaderEither<unknown, never, void>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export declare const flatMap: <A, R2, E2, B>(
  f: (a: A) => ReaderEither<R2, E2, B>
) => <R1, E1>(ma: ReaderEither<R1, E1, A>) => ReaderEither<R1 & R2, E1 | E2, B>
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <R1, E1, R2, E2, A>(
  mma: ReaderEither<R1, E1, ReaderEither<R2, E2, A>>
) => ReaderEither<R1 & R2, E1 | E2, A>
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category SemigroupK
 * @since 3.0.0
 */
export declare const combineK: <R2, E2, B>(
  second: LazyArg<ReaderEither<R2, E2, B>>
) => <R1, E1, A>(self: ReaderEither<R1, E1, A>) => ReaderEither<R1 & R2, E2, A | B>
/**
 * The default [`Applicative`](#applicative) instance returns the first error, if you want to
 * get all errors you need to provide an way to combine them via a `Semigroup`.
 *
 * See [`getValidatedApplicative`](./Either.ts.html#getvalidatedapplicative).
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getValidatedApplicative: <E>(
  S: Semigroup<E>
) => applicative.Applicative<either.Validatedλ<ReaderEitherλ, E>>
/**
 * The default [`SemigroupK`](#semigroupk) instance returns the last error, if you want to
 * get all errors you need to provide an way to combine them via a `Semigroup`.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getValidatedSemigroupK: <E>(
  S: Semigroup<E>
) => semigroupK.SemigroupK<either.Validatedλ<ReaderEitherλ, E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getCompactable: <E>(M: Monoid<E>) => compactable.Compactable<either.Validatedλ<ReaderEitherλ, E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFilterable: <E>(M: Monoid<E>) => filterable.Filterable<either.Validatedλ<ReaderEitherλ, E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<ReaderEitherλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <R, E, B>(fab: ReaderEither<R, E, (a: A) => B>) => ReaderEither<R, E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: pointed.Pointed<ReaderEitherλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: apply.Apply<ReaderEitherλ>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipLeftPar: <R2, E2, B>(
  second: ReaderEither<R2, E2, B>
) => <R1, E1, A>(self: ReaderEither<R1, E1, A>) => ReaderEither<R1 & R2, E1 | E2, A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipRightPar: <R2, E2, B>(
  second: ReaderEither<R2, E2, B>
) => <R1, E1, A>(self: ReaderEither<R1, E1, A>) => ReaderEither<R1 & R2, E1 | E2, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: applicative.Applicative<ReaderEitherλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Flattenable: flattenable.Flattenable<ReaderEitherλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: monad.Monad<ReaderEitherλ>
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const tap: <A, R2, E2, _>(
  f: (a: A) => ReaderEither<R2, E2, _>
) => <R1, E1>(self: ReaderEither<R1, E1, A>) => ReaderEither<R1 & R2, E1 | E2, A>
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export declare const tapError: <E1, R2, E2, _>(
  onError: (e: E1) => ReaderEither<R2, E2, _>
) => <R1, A>(self: ReaderEither<R1, E1, A>) => ReaderEither<R1 & R2, E1 | E2, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: bifunctor.Bifunctor<ReaderEitherλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupK: semigroupK.SemigroupK<ReaderEitherλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromReader: fromReader_.FromReader<ReaderEitherλ>
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const ask: <R>() => ReaderEither<R, never, R>
/**
 * Projects a value from the global context in a `ReaderEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const asks: <R, A>(f: (r: R) => A) => ReaderEither<R, never, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromReaderK: <A extends ReadonlyArray<unknown>, R, B>(
  f: (...a: A) => Reader<R, B>
) => (...a: A) => ReaderEither<R, never, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapReaderK: <A, R2, B>(
  f: (a: A) => Reader<R2, B>
) => <R1, E>(ma: ReaderEither<R1, E, A>) => ReaderEither<R1 & R2, E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: fromEither_.FromEither<ReaderEitherλ>
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromOption: <E>(onNone: LazyArg<E>) => <A>(fa: Option<A>) => ReaderEither<unknown, E, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <A extends ReadonlyArray<unknown>, B, E>(
  f: (...a: A) => Option<B>,
  onNone: (...a: A) => E
) => (...a: A) => ReaderEither<unknown, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapOptionK: <A, B, E>(
  f: (a: A) => Option<B>,
  onNone: (a: A) => E
) => <R>(ma: ReaderEither<R, E, A>) => ReaderEither<R, E, B>
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: {
  <C extends A, B extends A, E, A = C>(refinement: Refinement<A, B>, onFalse: (c: C) => E): (
    c: C
  ) => ReaderEither<unknown, E, B>
  <B extends A, E, A = B>(predicate: Predicate<A>, onFalse: (b: B) => E): (b: B) => ReaderEither<unknown, E, B>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const filter: {
  <C extends A, B extends A, E2, A = C>(refinement: Refinement<A, B>, onFalse: (c: C) => E2): <R, E1>(
    ma: ReaderEither<R, E1, C>
  ) => ReaderEither<R, E2 | E1, B>
  <B extends A, E2, A = B>(predicate: Predicate<A>, onFalse: (b: B) => E2): <R, E1>(
    mb: ReaderEither<R, E1, B>
  ) => ReaderEither<R, E2 | E1, B>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const filterMap: <A, B, E>(
  f: (a: A) => Option<B>,
  onNone: (a: A) => E
) => <R>(self: ReaderEither<R, E, A>) => ReaderEither<R, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const partition: {
  <C extends A, B extends A, E, A = C>(refinement: Refinement<A, B>, onFalse: (c: C) => E): <R>(
    self: ReaderEither<R, E, C>
  ) => readonly [ReaderEither<R, E, C>, ReaderEither<R, E, B>]
  <B extends A, E, A = B>(predicate: Predicate<A>, onFalse: (b: B) => E): <R>(
    self: ReaderEither<R, E, B>
  ) => readonly [ReaderEither<R, E, B>, ReaderEither<R, E, B>]
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const partitionMap: <A, B, C, E>(
  f: (a: A) => Either<B, C>,
  onEmpty: (a: A) => E
) => <R>(self: ReaderEither<R, E, A>) => readonly [ReaderEither<R, E, B>, ReaderEither<R, E, C>]
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => either.Either<E, B>
) => (...a: A) => ReaderEither<unknown, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapEitherK: <A, E2, B>(
  f: (a: A) => Either<E2, B>
) => <R, E1>(ma: ReaderEither<R, E1, A>) => ReaderEither<R, E1 | E2, B>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <E>(onNullable: LazyArg<E>) => <A>(a: A) => ReaderEither<unknown, E, NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <E>(
  onNullable: LazyArg<E>
) => <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => B | null | undefined
) => (...a: A) => ReaderEither<unknown, E, NonNullable<B>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const flatMapNullableK: <E>(
  onNullable: LazyArg<E>
) => <A, B>(f: (a: A) => B | null | undefined) => <R>(ma: ReaderEither<R, E, A>) => ReaderEither<R, E, NonNullable<B>>
/**
 * @since 3.0.0
 */
export declare const Do: ReaderEither<unknown, never, {}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <R, E, A>(
  fa: ReaderEither<R, E, A>
) => ReaderEither<
  R,
  E,
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <R, E>(
  fa: ReaderEither<R, E, A>
) => ReaderEither<
  R,
  E,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, R2, E2, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => ReaderEither<R2, E2, B>
) => <R1, E1>(
  fa: ReaderEither<R1, E1, A>
) => ReaderEither<
  R1 & R2,
  E1 | E2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const bindPar: <N extends string, A, R2, E2, B>(
  name: Exclude<N, keyof A>,
  fb: ReaderEither<R2, E2, B>
) => <R1, E1>(
  fa: ReaderEither<R1, E1, A>
) => ReaderEither<
  R1 & R2,
  E1 | E2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const ApT: ReaderEither<unknown, never, readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <R, E, A>(fa: ReaderEither<R, E, A>) => ReaderEither<R, E, readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <R2, E2, B>(
  fb: ReaderEither<R2, E2, B>
) => <R1, E1, A extends ReadonlyArray<unknown>>(
  fas: ReaderEither<R1, E1, A>
) => ReaderEither<R1 & R2, E1 | E2, readonly [...A, B]>
/**
 * Make sure that a resource is cleaned up in the event of an exception (\*). The release action is called regardless of
 * whether the body action throws (\*) or returns.
 *
 * (\*) i.e. returns a `Left`
 *
 * @since 3.0.0
 */
export declare const bracket: <R, E, A, B>(
  aquire: ReaderEither<R, E, A>,
  use: (a: A) => ReaderEither<R, E, B>,
  release: (a: A, e: Either<E, B>) => ReaderEither<R, E, void>
) => ReaderEither<R, E, B>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, R, E, B>(
  f: (index: number, a: A) => ReaderEither<R, E, B>
) => (as: readonly [A, ...A[]]) => ReaderEither<R, E, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, R, E, B>(
  f: (index: number, a: A) => ReaderEither<R, E, B>
) => (as: readonly A[]) => ReaderEither<R, E, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <A, R, E, B>(
  f: (a: A) => ReaderEither<R, E, B>
) => (as: readonly [A, ...A[]]) => ReaderEither<R, E, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <A, R, E, B>(
  f: (a: A) => ReaderEither<R, E, B>
) => (as: readonly A[]) => ReaderEither<R, E, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <R, E, A>(
  arr: ReadonlyArray<ReaderEither<R, E, A>>
) => ReaderEither<R, E, ReadonlyArray<A>>
