/**
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type { Apply } from './Apply'
import type * as bifunctor from './Bifunctor'
import type { Flattenable } from './Flattenable'
import type { FromIO } from './FromIO'
import type { FromReader } from './FromReader'
import type { FromTask } from './FromTask'
import * as fromWriter_ from './FromWriter'
import * as functor from './Functor'
import type { TypeLambda } from './HKT'
import type { IO } from './IO'
import type { Monad } from './Monad'
import type { Monoid } from './Monoid'
import type { Pointed } from './Pointed'
import type { Reader } from './Reader'
import * as readerTask from './ReaderTask'
import type { ReaderTask } from './ReaderTask'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Semigroup } from './Semigroup'
import type { Task } from './Task'
import type { Writer } from './Writer'
/**
 * @category model
 * @since 3.0.0
 */
export interface ReaderTaskWriter<R, W, A> extends Reader<R, Task<Writer<W, A>>> {}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ReaderTaskWriterλ extends TypeLambda {
  readonly type: ReaderTaskWriter<this['In1'], this['Out2'], this['Out1']>
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ReaderTaskWriterλFix<W> extends TypeLambda {
  readonly type: ReaderTaskWriter<this['In1'], W, this['Out1']>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromReader: <W>(w: W) => <R, A>(fa: Reader<R, A>) => ReaderTaskWriter<R, W, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromReaderTask: <W>(w: W) => <R, A>(a: ReaderTask<R, A>) => ReaderTaskWriter<R, W, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTaskWriter: <W, A>(a: Task<Writer<W, A>>) => ReaderTaskWriter<unknown, W, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromIO: <W>(w: W) => <A>(fa: IO<A>) => ReaderTaskWriter<unknown, W, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTask: <W>(w: W) => <A>(fa: Task<A>) => ReaderTaskWriter<unknown, W, A>
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const tell: <W, R>(w: W) => ReaderTaskWriter<R, W, void>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const asksReaderTaskWriter: <R1, R2, W, A>(
  f: (r1: R1) => ReaderTaskWriter<R2, W, A>
) => ReaderTaskWriter<R1 & R2, W, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromWriter: <W, A>(fa: Writer<W, A>) => ReaderTaskWriter<unknown, W, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromReaderWriter: <R, W, A>(fa: Reader<R, Writer<W, A>>) => ReaderTaskWriter<R, W, A>
/**
 * @since 3.0.0
 */
export declare const fst: <R, W, A>(self: ReaderTaskWriter<R, W, A>) => ReaderTask<R, W>
/**
 * @since 3.0.0
 */
export declare const snd: <R, W, A>(self: ReaderTaskWriter<R, W, A>) => ReaderTask<R, A>
/**
 * Alias of [`fst`](#fst).
 *
 * @since 3.0.0
 */
export declare const evaluate: <R, W, A>(self: ReaderTaskWriter<R, W, A>) => readerTask.ReaderTask<R, W>
/**
 * Alias of [`snd`](#snd).
 *
 * @since 3.0.0
 */
export declare const execute: <R, W, A>(self: ReaderTaskWriter<R, W, A>) => readerTask.ReaderTask<R, A>
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const local: <R2, R1>(
  f: (r2: R2) => R1
) => <W, A>(self: ReaderTaskWriter<R1, W, A>) => ReaderTaskWriter<R2, W, A>
/**
 * @since 3.0.0
 */
export declare const swap: <R, W, A>(self: ReaderTaskWriter<R, W, A>) => ReaderTaskWriter<R, A, W>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTaskWriterK: <A extends readonly unknown[], W, B>(
  f: (...a: A) => Task<Writer<W, B>>
) => (...a: A) => ReaderTaskWriter<unknown, W, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromReaderWriterK: <A extends readonly unknown[], R, W, B>(
  f: (...a: A) => Reader<R, Writer<W, B>>
) => (...a: A) => ReaderTaskWriter<R, W, B>
/**
 * @since 3.0.0
 */
export declare const listen: <R, W, A>(self: ReaderTaskWriter<R, W, A>) => ReaderTaskWriter<R, W, readonly [W, A]>
/**
 * @since 3.0.0
 */
export declare const pass: <R, W, A>(
  self: ReaderTaskWriter<R, W, readonly [A, (w: W) => W]>
) => ReaderTaskWriter<R, W, A>
/**
 * @since 3.0.0
 */
export declare const listens: <W, B>(
  f: (w: W) => B
) => <R, A>(self: ReaderTaskWriter<R, W, A>) => ReaderTaskWriter<R, W, readonly [A, B]>
/**
 * @since 3.0.0
 */
export declare const censor: <W>(f: (w: W) => W) => <R, A>(self: ReaderTaskWriter<R, W, A>) => ReaderTaskWriter<R, W, A>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <R, E>(self: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, E, B>
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapError: <E, G>(
  f: (e: E) => G
) => <R, A>(self: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, G, A>
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapBoth: <E, G, A, B>(
  f: (e: E) => G,
  g: (a: A) => B
) => <R>(self: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, G, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: bifunctor.Bifunctor<ReaderTaskWriterλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<ReaderTaskWriterλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(
  a: A
) => <R, E, B>(self: ReaderTaskWriter<R, E, (a: A) => B>) => ReaderTaskWriter<R, E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getPointed: <W>(M: Monoid<W>) => Pointed<ReaderTaskWriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <W>(A: Apply<readerTask.ReaderTaskλ>, S: Semigroup<W>) => Apply<ReaderTaskWriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <W>(
  A: Apply<readerTask.ReaderTaskλ>,
  M: Monoid<W>
) => Applicative<ReaderTaskWriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFlattenable: <W>(S: Semigroup<W>) => Flattenable<ReaderTaskWriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <W>(M: Monoid<W>) => Monad<ReaderTaskWriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromWriter: fromWriter_.FromWriter<ReaderTaskWriterλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromWriterK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => Writer<E, B>
) => (...a: A) => ReaderTaskWriter<unknown, E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFromReader: <W>(M: Monoid<W>) => FromReader<ReaderTaskWriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFromIO: <W>(M: Monoid<W>) => FromIO<ReaderTaskWriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFromTask: <W>(M: Monoid<W>) => FromTask<ReaderTaskWriterλFix<W>>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <R, E, A>(
  self: ReaderTaskWriter<R, E, A>
) => ReaderTaskWriter<
  R,
  E,
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <R, E>(
  fa: ReaderTaskWriter<R, E, A>
) => ReaderTaskWriter<
  R,
  E,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const tupled: <R, E, A>(self: ReaderTaskWriter<R, E, A>) => ReaderTaskWriter<R, E, readonly [A]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(A, M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <W>(
  A: Apply<readerTask.ReaderTaskλ>,
  S: Semigroup<W>
) => <A, R, B>(
  f: (index: number, a: A) => ReaderTaskWriter<R, W, B>
) => (as: readonly [A, ...A[]]) => ReaderTaskWriter<R, W, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(A, M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <W>(
  A: Apply<readerTask.ReaderTaskλ>,
  M: Monoid<W>
) => <A, R, B>(
  f: (index: number, a: A) => ReaderTaskWriter<R, W, B>
) => (as: readonly A[]) => ReaderTaskWriter<R, W, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(A, M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <W>(
  A: Apply<readerTask.ReaderTaskλ>,
  S: Semigroup<W>
) => <A, R, B>(
  f: (a: A) => ReaderTaskWriter<R, W, B>
) => (as: readonly [A, ...A[]]) => ReaderTaskWriter<R, W, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(A, M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <W>(
  A: Apply<readerTask.ReaderTaskλ>,
  M: Monoid<W>
) => <A, R, B>(f: (a: A) => ReaderTaskWriter<R, W, B>) => (as: readonly A[]) => ReaderTaskWriter<R, W, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(A, M))`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <W>(
  A: Apply<readerTask.ReaderTaskλ>,
  M: Monoid<W>
) => <R, A>(arr: readonly ReaderTaskWriter<R, W, A>[]) => ReaderTaskWriter<R, W, readonly A[]>
