/**
 * The `Show` type class represents those types which can be converted into
 * a human-readable `string` representation.
 *
 * While not required, it is recommended that for any expression `x`, the
 * string `show(x)` be executable TypeScript code which evaluates to the same
 * value as the expression `x`.
 *
 * @since 3.0.0
 */
import type * as contravariant from './Contravariant'
import type { TypeLambda } from './HKT'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Show<A> {
  readonly show: (a: A) => string
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface Showλ extends TypeLambda {
  readonly type: Show<this['In1']>
}
/**
 * @since 3.0.0
 */
export declare const contramap: <B, A>(f: (b: B) => A) => (self: Show<A>) => Show<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Contravariant: contravariant.Contravariant<Showλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const struct: <A>(shows: { [K in keyof A]: Show<A[K]> }) => Show<{ readonly [K_1 in keyof A]: A[K_1] }>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const tuple: <A extends readonly unknown[]>(
  ...shows: { [K in keyof A]: Show<A[K]> }
) => Show<Readonly<A>>
