/**
 * @since 3.0.0
 */
import type { Comonad as Comonad_ } from './Comonad'
import type { Endomorphism } from './Endomorphism'
import * as functor from './Functor'
import type { TypeLambda, Kind } from './HKT'
/**
 * @category model
 * @since 3.0.0
 */
export interface Store<S, A> {
  readonly peek: (s: S) => A
  readonly pos: S
}
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <S>(fa: Store<S, A>) => Store<S, B>
/**
 * @category Extendable
 * @since 3.0.0
 */
export declare const extend: <S, A, B>(f: (wa: Store<S, A>) => B) => (wa: Store<S, A>) => Store<S, B>
/**
 * @category Extract
 * @since 3.0.0
 */
export declare const extract: <S, A>(wa: Store<S, A>) => A
/**
 * Derivable from `Extendable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const duplicate: <S, A>(wa: Store<S, A>) => Store<S, Store<S, A>>
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface Storeλ extends TypeLambda {
  readonly type: Store<this['InOut1'], this['Out1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<Storeλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <S, B>(fab: Store<S, (a: A) => B>) => Store<S, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: Comonad_<Storeλ>
/**
 * Reposition the focus at the specified position.
 *
 * @since 3.0.0
 */
export declare const seek: <S>(s: S) => <A>(wa: Store<S, A>) => Store<S, A>
/**
 * Reposition the focus at the specified position, which depends on the current position.
 *
 * @since 3.0.0
 */
export declare const seeks: <S>(f: Endomorphism<S>) => <A>(wa: Store<S, A>) => Store<S, A>
/**
 * Extract a value from a position which depends on the current position.
 *
 * @since 3.0.0
 */
export declare const peeks: <S>(f: Endomorphism<S>) => <A>(wa: Store<S, A>) => A
/**
 * Extract a collection of values from positions which depend on the current position.
 *
 * @since 3.0.0
 */
export declare function experiment<F extends TypeLambda>(
  F: functor.Functor<F>
): <S1, S2, R, W, E>(f: (s: S1) => Kind<F, S2, R, W, E, S1>) => <A>(wa: Store<S1, A>) => Kind<F, S2, R, W, E, A>
