/**
 * @since 3.0.0
 */
import type * as semigroupK from './SemigroupK'
import * as monoidK from './MonoidK'
import type { Applicative } from './Applicative'
import type { Apply } from './Apply'
import * as flattenable from './Flattenable'
import * as compactable from './Compactable'
import type { Either } from './Either'
import * as filterable from './Filterable'
import * as fromOption_ from './FromOption'
import * as fromEither_ from './FromEither'
import * as fromIO_ from './FromIO'
import * as formTask_ from './FromTask'
import type { LazyArg } from './function'
import * as functor from './Functor'
import type { TypeLambda } from './HKT'
import type { IO } from './IO'
import type { IOEither } from './IOEither'
import type * as monad from './Monad'
import * as option from './Option'
import type { Option } from './Option'
import type * as pointed from './Pointed'
import type { Predicate } from './Predicate'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Refinement } from './Refinement'
import * as task from './Task'
import type { Task } from './Task'
import type { TaskEither } from './TaskEither'
/**
 * @category model
 * @since 3.0.0
 */
export interface TaskOption<A> extends Task<Option<A>> {}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const some: <A>(a: A) => TaskOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromOption: <A>(fa: Option<A>) => TaskOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromEither: <A>(fa: Either<unknown, A>) => TaskOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIO: <A>(fa: IO<A>) => TaskOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromTask: <A>(fa: task.Task<A>) => TaskOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIOEither: <A>(fa: IOEither<unknown, A>) => TaskOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromTaskEither: <A>(fa: TaskEither<unknown, A>) => TaskOption<A>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <B, A, C = B>(onNone: LazyArg<B>, onSome: (a: A) => C) => (ma: TaskOption<A>) => Task<B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const matchWithEffect: <B, A, C = B>(
  onNone: LazyArg<Task<B>>,
  onSome: (a: A) => Task<C>
) => (ma: TaskOption<A>) => Task<B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElse: <B>(onNone: LazyArg<B>) => <A>(ma: TaskOption<A>) => Task<A | B>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElseE: <B>(onNone: LazyArg<Task<B>>) => <A>(ma: TaskOption<A>) => Task<A | B>
/**
 * Transforms a `Promise` that may reject to a `Promise` that never rejects and returns an `Option` instead.
 *
 * Note: `f` should never `throw` errors, they are not caught.
 *
 * See also [`tryCatchK`](#tryCatchK).
 *
 * @category interop
 * @since 3.0.0
 */
export declare const tryCatch: <A>(f: LazyArg<Promise<A>>) => TaskOption<A>
/**
 * Converts a function returning a `Promise` to one returning a `TaskOption`.
 *
 * @category interop
 * @since 3.0.0
 */
export declare const tryCatchK: <A extends readonly unknown[], B>(
  f: (...a: A) => Promise<B>
) => (...a: A) => TaskOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTaskEitherK: <A extends readonly unknown[], B>(
  f: (...a: A) => TaskEither<unknown, B>
) => (...a: A) => TaskOption<B>
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: TaskOption<A>) => TaskOption<B>
/**
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <A>(fa: TaskOption<A>) => <B>(fab: TaskOption<(a: A) => B>) => TaskOption<B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => TaskOption<A>
/**
 * @since 3.0.0
 */
export declare const unit: TaskOption<void>
/**
 * @category Flattenable
 * @since 3.0.0
 */
export declare const flatMap: <A, B>(f: (a: A) => TaskOption<B>) => (ma: TaskOption<A>) => TaskOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapTaskEitherK: <A, B>(
  f: (a: A) => TaskEither<unknown, B>
) => (ma: TaskOption<A>) => TaskOption<B>
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: TaskOption<TaskOption<A>>) => TaskOption<A>
/**
 * @category SemigroupK
 * @since 3.0.0
 */
export declare const combineK: <B>(second: LazyArg<TaskOption<B>>) => <A>(self: TaskOption<A>) => TaskOption<A | B>
/**
 * @category MonoidK
 * @since 3.0.0
 */
export declare const emptyK: <A>() => TaskOption<A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const none: TaskOption<never>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: <A>(foa: TaskOption<option.Option<A>>) => TaskOption<A>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: <A, B>(fe: TaskOption<Either<A, B>>) => readonly [TaskOption<A>, TaskOption<B>]
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: <A, B>(f: (a: A) => option.Option<B>) => (fa: TaskOption<A>) => TaskOption<B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: <A, B, C>(
  f: (a: A) => Either<B, C>
) => (fa: TaskOption<A>) => readonly [TaskOption<B>, TaskOption<C>]
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface TaskOptionλ extends TypeLambda {
  readonly type: TaskOption<this['Out1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<TaskOptionλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: TaskOption<(a: A) => B>) => TaskOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: pointed.Pointed<TaskOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const ApplyPar: Apply<TaskOptionλ>
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipLeftPar: <B>(second: TaskOption<B>) => <A>(self: TaskOption<A>) => TaskOption<A>
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipRightPar: <B>(second: TaskOption<B>) => <A>(self: TaskOption<A>) => TaskOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const ApplicativePar: Applicative<TaskOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Flattenable: flattenable.Flattenable<TaskOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const ApplySeq: Apply<TaskOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const ApplicativeSeq: Applicative<TaskOptionλ>
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const tap: <A, _>(f: (a: A) => TaskOption<_>) => (self: TaskOption<A>) => TaskOption<A>
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export declare const tapError: <_>(onNone: () => TaskOption<_>) => <A>(self: TaskOption<A>) => TaskOption<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: monad.Monad<TaskOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupK: semigroupK.SemigroupK<TaskOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const MonoidK: monoidK.MonoidK<TaskOptionλ>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const guard: (b: boolean) => TaskOption<void>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: fromIO_.FromIO<TaskOptionλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => IO<B>) => (...a: A) => TaskOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapIOK: <A, B>(f: (a: A) => IO<B>) => (self: TaskOption<A>) => TaskOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromOption: fromOption_.FromOption<TaskOptionλ>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: <B extends A, A = B>(predicate: Predicate<A>) => (b: B) => TaskOption<B>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromRefinement: <C extends A, B extends A, A = C>(
  refinement: Refinement<A, B>
) => (c: C) => TaskOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => Option<B>
) => (...a: A) => TaskOption<B>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <A>(a: A) => TaskOption<NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => B | null | undefined
) => (...a: A) => TaskOption<NonNullable<B>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const flatMapNullableK: <A, B>(
  f: (a: A) => B | null | undefined
) => (ma: TaskOption<A>) => TaskOption<NonNullable<B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: fromEither_.FromEither<TaskOptionλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => Either<E, B>
) => (...a: A) => TaskOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapEitherK: <A, E, B>(f: (a: A) => Either<E, B>) => (ma: TaskOption<A>) => TaskOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromTask: formTask_.FromTask<TaskOptionλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTaskK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => task.Task<B>
) => (...a: A) => TaskOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapTaskK: <A, B>(f: (a: A) => task.Task<B>) => (self: TaskOption<A>) => TaskOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: compactable.Compactable<TaskOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: filterable.Filterable<TaskOptionλ>
/**
 * @since 3.0.0
 */
export declare const filter: {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): (fc: TaskOption<C>) => TaskOption<B>
  <B extends A, A = B>(predicate: Predicate<A>): (fb: TaskOption<B>) => TaskOption<B>
}
/**
 * @since 3.0.0
 */
export declare const partition: {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): (
    fc: TaskOption<C>
  ) => readonly [TaskOption<C>, TaskOption<B>]
  <B extends A, A = B>(predicate: Predicate<A>): (fb: TaskOption<B>) => readonly [TaskOption<B>, TaskOption<B>]
}
/**
 * @since 3.0.0
 */
export declare const Do: TaskOption<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(fa: TaskOption<A>) => TaskOption<{
  readonly [K in N]: A
}>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (fa: TaskOption<A>) => TaskOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => TaskOption<B>
) => (ma: TaskOption<A>) => TaskOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
/**
 * @since 3.0.0
 */
export declare const bindPar: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: TaskOption<B>
) => (fa: TaskOption<A>) => TaskOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
/**
 * @since 3.0.0
 */
export declare const ApT: TaskOption<readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <A>(fa: TaskOption<A>) => TaskOption<readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <B>(
  fb: TaskOption<B>
) => <A extends ReadonlyArray<unknown>>(fas: TaskOption<A>) => TaskOption<readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplyPar)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => TaskOption<B>
) => (as: readonly [A, ...A[]]) => TaskOption<readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => TaskOption<B>
) => (as: readonly A[]) => TaskOption<readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplyPar)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <A, B>(
  f: (a: A) => TaskOption<B>
) => (as: readonly [A, ...A[]]) => TaskOption<readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <A, B>(
  f: (a: A) => TaskOption<B>
) => (as: readonly A[]) => TaskOption<readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <A>(arr: ReadonlyArray<TaskOption<A>>) => TaskOption<ReadonlyArray<A>>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplySeq)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndexSeq: <A, B>(
  f: (index: number, a: A) => TaskOption<B>
) => (as: readonly [A, ...A[]]) => TaskOption<readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndexSeq: <A, B>(
  f: (index: number, a: A) => TaskOption<B>
) => (as: readonly A[]) => TaskOption<readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplySeq)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArraySeq: <A, B>(
  f: (a: A) => TaskOption<B>
) => (as: readonly [A, ...A[]]) => TaskOption<readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArraySeq: <A, B>(
  f: (a: A) => TaskOption<B>
) => (as: readonly A[]) => TaskOption<readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArraySeq: <A>(arr: ReadonlyArray<TaskOption<A>>) => TaskOption<ReadonlyArray<A>>
