/**
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type { Apply } from './Apply'
import type * as bifunctor from './Bifunctor'
import type { Flattenable } from './Flattenable'
import type { Either, Validatedλ } from './Either'
import * as fromEither_ from './FromEither'
import * as fromIO_ from './FromIO'
import * as fromTask_ from './FromTask'
import * as fromThese_ from './FromThese'
import type { LazyArg } from './function'
import * as functor from './Functor'
import type { TypeLambda } from './HKT'
import type { IO } from './IO'
import type { IOEither } from './IOEither'
import type { Monad } from './Monad'
import type { Option } from './Option'
import type * as pointed from './Pointed'
import type { Predicate } from './Predicate'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Refinement } from './Refinement'
import type { Semigroup } from './Semigroup'
import * as task from './Task'
import type { Task } from './Task'
import * as these from './These'
import type { These } from './These'
/**
 * @category model
 * @since 3.0.0
 */
export interface TaskThese<E, A> extends Task<These<E, A>> {}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface TaskTheseλ extends TypeLambda {
  readonly type: TaskThese<this['Out2'], this['Out1']>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const left: <E>(e: E) => TaskThese<E, never>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const right: <A>(a: A) => TaskThese<never, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const both: <E, A>(e: E, a: A) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const rightTask: <A>(ma: Task<A>) => TaskThese<never, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const leftTask: <E>(me: Task<E>) => TaskThese<E, never>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const rightIO: <A>(ma: IO<A>) => TaskThese<never, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const leftIO: <E>(me: IO<E>) => TaskThese<E, never>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIOEither: <E, A>(fa: IOEither<E, A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromEither: <E, A>(fa: Either<E, A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromThese: <E, A>(fa: these.These<E, A>) => TaskThese<E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromIO: <A>(fa: IO<A>) => TaskThese<never, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTask: <A>(fa: task.Task<A>) => TaskThese<never, A>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <E, B, A, C = B, D = B>(
  onError: (e: E) => B,
  onSuccess: (a: A) => C,
  onBoth: (e: E, a: A) => D
) => (ma: task.Task<these.These<E, A>>) => task.Task<B | C | D>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const matchWithEffect: <E, B, A, C = B, D = B>(
  onError: (e: E) => task.Task<B>,
  onSuccess: (a: A) => task.Task<C>,
  onBoth: (e: E, a: A) => task.Task<D>
) => (ma: task.Task<these.These<E, A>>) => task.Task<B | C | D>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const swap: <E, A>(ma: task.Task<these.These<E, A>>) => task.Task<these.These<A, E>>
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <E>(fa: TaskThese<E, A>) => TaskThese<E, B>
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapBoth: <E, G, A, B>(f: (e: E) => G, g: (a: A) => B) => (self: TaskThese<E, A>) => TaskThese<G, B>
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapError: <E, G>(f: (e: E) => G) => <A>(self: TaskThese<E, A>) => TaskThese<G, A>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => TaskThese<never, A>
/**
 * @since 3.0.0
 */
export declare const unit: TaskThese<never, void>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <E>(A: Apply<task.Taskλ>, S: Semigroup<E>) => Apply<Validatedλ<TaskTheseλ, E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <E>(
  A: Apply<task.Taskλ>,
  S: Semigroup<E>
) => Applicative<Validatedλ<TaskTheseλ, E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFlattenable: <E>(S: Semigroup<E>) => Flattenable<Validatedλ<TaskTheseλ, E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <E>(S: Semigroup<E>) => Monad<Validatedλ<TaskTheseλ, E>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<TaskTheseλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: TaskThese<E, (a: A) => B>) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: pointed.Pointed<TaskTheseλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: bifunctor.Bifunctor<TaskTheseλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: fromEither_.FromEither<TaskTheseλ>
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromOption: <E>(onNone: LazyArg<E>) => <A>(fa: Option<A>) => TaskThese<E, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <A extends ReadonlyArray<unknown>, B, E>(
  f: (...a: A) => Option<B>,
  onNone: (...a: A) => E
) => (...a: A) => TaskThese<E, B>
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: {
  <C extends A, B extends A, E, A = C>(refinement: Refinement<A, B>, onFalse: (c: C) => E): (c: C) => TaskThese<E, B>
  <B extends A, E, A = B>(predicate: Predicate<A>, onFalse: (b: B) => E): (b: B) => TaskThese<E, B>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => Either<E, B>
) => (...a: A) => TaskThese<E, B>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <E>(onNullable: LazyArg<E>) => <A>(a: A) => TaskThese<E, NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <E>(
  onNullable: LazyArg<E>
) => <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => B | null | undefined
) => (...a: A) => TaskThese<E, NonNullable<B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromThese: fromThese_.FromThese<TaskTheseλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTheseK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => these.These<E, B>
) => (...a: A) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: fromIO_.FromIO<TaskTheseλ>
/**
 * @category logging
 * @since 3.0.0
 */
export declare const log: (...x: ReadonlyArray<unknown>) => TaskThese<never, void>
/**
 * @category logging
 * @since 3.0.0
 */
export declare const logError: (...x: ReadonlyArray<unknown>) => TaskThese<never, void>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => IO<B>
) => <E>(...a: A) => TaskThese<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromTask: fromTask_.FromTask<TaskTheseλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromTaskK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => task.Task<B>
) => (...a: A) => TaskThese<never, B>
/**
 * @since 3.0.0
 */
export declare const toTuple2: <E, A>(
  e: LazyArg<E>,
  a: LazyArg<A>
) => (fa: TaskThese<E, A>) => task.Task<readonly [E, A]>
/**
 * @since 3.0.0
 */
export declare const ApT: TaskThese<never, readonly []>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(T.ApplyPar, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <E>(
  S: Semigroup<E>
) => <A, B>(
  f: (index: number, a: A) => TaskThese<E, B>
) => (as: readonly [A, ...A[]]) => TaskThese<E, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <E>(
  S: Semigroup<E>
) => <A, B>(f: (index: number, a: A) => TaskThese<E, B>) => (as: readonly A[]) => TaskThese<E, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(T.ApplyPar, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <E>(
  S: Semigroup<E>
) => <A, B>(f: (a: A) => TaskThese<E, B>) => (as: readonly [A, ...A[]]) => TaskThese<E, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <E>(
  S: Semigroup<E>
) => <A, B>(f: (a: A) => TaskThese<E, B>) => (as: readonly A[]) => TaskThese<E, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(T.ApplicativePar, S))`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <E>(
  S: Semigroup<E>
) => <A>(arr: readonly TaskThese<E, A>[]) => TaskThese<E, readonly A[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(T.ApplySeq, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndexSeq: <E>(
  S: Semigroup<E>
) => <A, B>(
  f: (index: number, a: A) => TaskThese<E, B>
) => (as: readonly [A, ...A[]]) => TaskThese<E, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndexSeq: <E>(
  S: Semigroup<E>
) => <A, B>(f: (index: number, a: A) => TaskThese<E, B>) => (as: readonly A[]) => TaskThese<E, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(T.ApplySeq, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArraySeq: <E>(
  S: Semigroup<E>
) => <A, B>(f: (a: A) => TaskThese<E, B>) => (as: readonly [A, ...A[]]) => TaskThese<E, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArraySeq: <E>(
  S: Semigroup<E>
) => <A, B>(f: (a: A) => TaskThese<E, B>) => (as: readonly A[]) => TaskThese<E, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(T.ApplicativeSeq, S))`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArraySeq: <E>(
  S: Semigroup<E>
) => <A>(arr: readonly TaskThese<E, A>[]) => TaskThese<E, readonly A[]>
