/**
 * @since 3.0.0
 */
import type { Apply } from './Apply'
import type { Flattenable } from './Flattenable'
import type { LazyArg } from './function'
import type { Functor } from './Functor'
import type { TypeLambda, Kind } from './HKT'
import type { Monad } from './Monad'
import type { Pointed } from './Pointed'
import type { Semigroup } from './Semigroup'
import * as T from './These'
import type { These } from './These'
/**
 * @since 3.0.0
 */
export declare const right: <F extends TypeLambda>(
  F: Pointed<F>
) => <A, S>(a: A) => Kind<F, S, unknown, never, never, T.These<never, A>>
/**
 * @since 3.0.0
 */
export declare const left: <F extends TypeLambda>(
  F: Pointed<F>
) => <E, S>(e: E) => Kind<F, S, unknown, never, never, T.These<E, never>>
/**
 * @since 3.0.0
 */
export declare const both: <F extends TypeLambda>(
  F: Pointed<F>
) => <E, A, S>(e: E, a: A) => Kind<F, S, unknown, never, never, T.These<E, A>>
/**
 * @since 3.0.0
 */
export declare function rightF<F extends TypeLambda>(
  F: Functor<F>
): <S, R, W, E, A>(fa: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, These<never, A>>
/**
 * @since 3.0.0
 */
export declare function leftF<F extends TypeLambda>(
  F: Functor<F>
): <S, R, W, E, L>(fl: Kind<F, S, R, W, E, L>) => Kind<F, S, R, W, E, These<L, never>>
/**
 * @since 3.0.0
 */
export declare function map<F extends TypeLambda>(
  F: Functor<F>
): <A, B>(
  f: (a: A) => B
) => <S, R, W, FE, E>(self: Kind<F, S, R, W, FE, These<E, A>>) => Kind<F, S, R, W, FE, These<E, B>>
/**
 * @since 3.0.0
 */
export declare const ap: <F extends TypeLambda, E>(
  F: Apply<F>,
  S: Semigroup<E>
) => <S, R2, W2, FE2, A>(
  fa: Kind<F, S, R2, W2, FE2, T.These<E, A>>
) => <R1, W1, FE1, B>(
  self: Kind<F, S, R1, W1, FE1, T.These<E, (a: A) => B>>
) => Kind<F, S, R1 & R2, W2 | W1, FE2 | FE1, T.These<E, B>>
/**
 * @since 3.0.0
 */
export declare const flatMap: <M extends TypeLambda, E>(
  M: Monad<M>,
  S: Semigroup<E>
) => <A, S, R2, W2, FE2, B>(
  f: (a: A) => Kind<M, S, R2, W2, FE2, T.These<E, B>>
) => <R1, W1, FE1>(
  self: Kind<M, S, R1, W1, FE1, T.These<E, A>>
) => Kind<M, S, R1 & R2, W2 | W1, FE2 | FE1, T.These<E, B>>
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapBoth: <F extends TypeLambda>(
  F: Functor<F>
) => <E, G, A, B>(
  f: (e: E) => G,
  g: (a: A) => B
) => <S, R, W, FE>(self: Kind<F, S, R, W, FE, T.These<E, A>>) => Kind<F, S, R, W, FE, T.These<G, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapLeft: <F extends TypeLambda>(
  F: Functor<F>
) => <E, G>(
  f: (e: E) => G
) => <S, R, W, FE, A>(self: Kind<F, S, R, W, FE, T.These<E, A>>) => Kind<F, S, R, W, FE, T.These<G, A>>
/**
 * @since 3.0.0
 */
export declare function match<F extends TypeLambda>(
  F: Functor<F>
): <E, B, A, C = B, D = B>(
  onError: (e: E) => B,
  onSuccess: (a: A) => C,
  onBoth: (e: E, a: A) => D
) => <S, R, W, FE>(self: Kind<F, S, R, W, FE, These<E, A>>) => Kind<F, S, R, W, FE, B | C | D>
/**
 * @since 3.0.0
 */
export declare const matchWithEffect: <M extends TypeLambda>(
  M: Flattenable<M>
) => <E, S, R2, W2, FE2, B, A, R3, W3, FE3, R4, W4, FE4, C = B, D = B>(
  onError: (e: E) => Kind<M, S, R2, W2, FE2, B>,
  onSuccess: (a: A) => Kind<M, S, R3, W3, FE3, C>,
  onBoth: (e: E, a: A) => Kind<M, S, R4, W4, FE4, D>
) => <R1, W1, FE1>(
  self: Kind<M, S, R1, W1, FE1, T.These<E, A>>
) => Kind<M, S, R1 & R2 & R3 & R4, W2 | W3 | W4 | W1, FE2 | FE3 | FE4 | FE1, B | C | D>
/**
 * @since 3.0.0
 */
export declare function swap<F extends TypeLambda>(
  F: Functor<F>
): <S, R, W, FE, E, A>(self: Kind<F, S, R, W, FE, These<E, A>>) => Kind<F, S, R, W, FE, These<A, E>>
/**
 * @since 3.0.0
 */
export declare function toTuple2<F extends TypeLambda>(
  F: Functor<F>
): <E, A>(
  e: LazyArg<E>,
  a: LazyArg<A>
) => <S, R, W, FE>(self: Kind<F, S, R, W, FE, These<E, A>>) => Kind<F, S, R, W, FE, readonly [E, A]>
