/**
 * @since 3.0.0
 */
import type { Comonad } from './Comonad'
import * as functor from './Functor'
import type { TypeLambda } from './HKT'
import type { Monoid } from './Monoid'
/**
 * @category model
 * @since 3.0.0
 */
export interface Traced<W, A> {
  (w: W): A
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface Tracedλ extends TypeLambda {
  readonly type: Traced<this['In1'], this['Out1']>
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface TracedλFix<W> extends TypeLambda {
  readonly type: Traced<W, this['Out1']>
}
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <W>(self: Traced<W, A>) => Traced<W, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<Tracedλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <W, B>(self: Traced<W, (a: A) => B>) => Traced<W, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getComonad: <W>(M: Monoid<W>) => Comonad<TracedλFix<W>>
/**
 * Extracts a value at a relative position which depends on the current value.
 *
 * @since 3.0.0
 */
export declare const tracks: <W>(M: Monoid<W>) => <A>(f: (a: A) => W) => (fa: Traced<W, A>) => A
/**
 * Get the current position.
 *
 * @since 3.0.0
 */
export declare const listen: <W, A>(fa: Traced<W, A>) => Traced<W, readonly [A, W]>
/**
 * Get a value which depends on the current position.
 *
 * @since 3.0.0
 */
export declare const listens: <W, B>(f: (w: W) => B) => <A>(pa: Traced<W, A>) => Traced<W, readonly [A, B]>
/**
 * Apply a function to the current position.
 *
 * @since 3.0.0
 */
export declare const censor: <W>(f: (p: W) => W) => <A>(pa: Traced<W, A>) => Traced<W, A>
