/**
 * `Traversable` represents data structures which can be _traversed_ accumulating results and effects in some
 * `Applicative` functor.
 *
 * - `traverse` runs an action for every element in a data structure, and accumulates the results
 *
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type { TypeLambda, Kind, TypeClass } from './HKT'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Traversable<F extends TypeLambda> extends TypeClass<F> {
  readonly traverse: <G extends TypeLambda>(
    G: Applicative<G>
  ) => <A, S, R, W, E, B>(
    f: (a: A) => Kind<G, S, R, W, E, B>
  ) => <FS, FR, FW, FE>(self: Kind<F, FS, FR, FW, FE, A>) => Kind<G, S, R, W, E, Kind<F, FS, FR, FW, FE, B>>
}
/**
 * Return a default `sequence` implementation from `traverse`.
 *
 * @category defaults
 * @since 3.0.0
 */
export declare const getDefaultSequence: <F extends TypeLambda>(
  traverse: <G extends TypeLambda>(
    G: Applicative<G>
  ) => <A, S, R, W, E, B>(
    f: (a: A) => Kind<G, S, R, W, E, B>
  ) => <FS, FR, FW, FE>(self: Kind<F, FS, FR, FW, FE, A>) => Kind<G, S, R, W, E, Kind<F, FS, FR, FW, FE, B>>
) => <G_1 extends TypeLambda>(
  G: Applicative<G_1>
) => <FS_1, FR_1, FW_1, FE_1, S_1, R_1, W_1, E_1, A_1>(
  fa: Kind<F, FS_1, FR_1, FW_1, FE_1, Kind<G_1, S_1, R_1, W_1, E_1, A_1>>
) => Kind<G_1, S_1, R_1, W_1, E_1, Kind<F, FS_1, FR_1, FW_1, FE_1, A_1>>
/**
 * `traverse` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const getTraverseComposition: <F extends TypeLambda, G extends TypeLambda>(
  F: Traversable<F>,
  G: Traversable<G>
) => <H extends TypeLambda>(
  H: Applicative<H>
) => <A, S, R, W, E, B>(
  f: (a: A) => Kind<H, S, R, W, E, B>
) => <FS, FR, FW, FE, GS, GR, GW, GE>(
  fga: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>
) => Kind<H, S, R, W, E, Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, B>>>
