/**
 * @since 3.0.0
 */
import type { Apply } from './Apply'
import type { Flattenable } from './Flattenable'
import type { FromIO } from './FromIO'
import type { FromTask } from './FromTask'
import type { Functor } from './Functor'
import type { TypeLambda, Kind } from './HKT'
import type { IO } from './IO'
import type { Monoid } from './Monoid'
import type { Pointed } from './Pointed'
import type { Semigroup } from './Semigroup'
import type { Task } from './Task'
import * as writer from './Writer'
import type { Writer } from './Writer'
/**
 * @category constructors
 * @since 3.0.0
 */
export declare function fromF<F extends TypeLambda>(
  F: Functor<F>
): <W>(w: W) => <S, R, FW, E, A>(fa: Kind<F, S, R, FW, E, A>) => Kind<F, S, R, FW, E, Writer<W, A>>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromIO: <F extends TypeLambda>(
  F: Functor<F>,
  FT: FromIO<F>
) => <W>(w: W) => <A, S>(fa: IO<A>) => Kind<F, S, unknown, never, never, writer.Writer<W, A>>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromTask: <F extends TypeLambda>(
  F: Functor<F>,
  FT: FromTask<F>
) => <W>(w: W) => <A, S>(fa: Task<A>) => Kind<F, S, unknown, never, never, writer.Writer<W, A>>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const tell: <F extends TypeLambda>(
  F: Pointed<F>
) => <W, S>(w: W) => Kind<F, S, unknown, never, never, writer.Writer<W, void>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare function map<F extends TypeLambda>(
  F: Functor<F>
): <A, B>(
  f: (a: A) => B
) => <S, R, FW, E, W>(self: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<W, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare function of<F extends TypeLambda, W>(
  F: Pointed<F>,
  M: Monoid<W>
): <A, S>(a: A) => Kind<F, S, unknown, never, never, Writer<W, A>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const ap: <F extends TypeLambda, W>(
  F: Apply<F>,
  S: Semigroup<W>
) => <S, R2, FW2, E2, A>(
  fa: Kind<F, S, R2, FW2, E2, writer.Writer<W, A>>
) => <R1, FW1, E1, B>(
  self: Kind<F, S, R1, FW1, E1, writer.Writer<W, (a: A) => B>>
) => Kind<F, S, R1 & R2, FW2 | FW1, E2 | E1, writer.Writer<W, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const flatMap: <M extends TypeLambda, W>(
  M: Flattenable<M>,
  S: Semigroup<W>
) => <A, S, R1, FW1, E1, B>(
  f: (a: A) => Kind<M, S, R1, FW1, E1, writer.Writer<W, B>>
) => <R2, FW2, E2>(
  self: Kind<M, S, R2, FW2, E2, writer.Writer<W, A>>
) => Kind<M, S, R1 & R2, FW1 | FW2, E1 | E2, writer.Writer<W, B>>
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapBoth: <F extends TypeLambda>(
  F: Functor<F>
) => <W, G, A, B>(
  f: (w: W) => G,
  g: (a: A) => B
) => <S, R, FW, E>(self: Kind<F, S, R, FW, E, writer.Writer<W, A>>) => Kind<F, S, R, FW, E, writer.Writer<G, B>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapLeft: <F extends TypeLambda>(
  F: Functor<F>
) => <W, G>(
  f: (w: W) => G
) => <S, R, FW, E, A>(self: Kind<F, S, R, FW, E, writer.Writer<W, A>>) => Kind<F, S, R, FW, E, writer.Writer<G, A>>
/**
 * @since 3.0.0
 */
export declare function fst<F extends TypeLambda>(
  F: Functor<F>
): <S, R, FW, E, W, A>(self: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, W>
/**
 * @since 3.0.0
 */
export declare function snd<F extends TypeLambda>(
  F: Functor<F>
): <S, R, FW, E, W, A>(self: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare function swap<F extends TypeLambda>(
  F: Functor<F>
): <S, R, FW, E, W, A>(self: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<A, W>>
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function listen<F extends TypeLambda>(
  F: Functor<F>
): <S, R, FW, E, W, A>(self: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<W, readonly [W, A]>>
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function pass<F extends TypeLambda>(
  F: Functor<F>
): <S, R, FW, E, W, A>(
  self: Kind<F, S, R, FW, E, Writer<W, readonly [A, (w: W) => W]>>
) => Kind<F, S, R, FW, E, Writer<W, A>>
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function listens<F extends TypeLambda>(
  F: Functor<F>
): <W, B>(
  f: (w: W) => B
) => <S, R, FW, E, A>(self: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<W, readonly [A, B]>>
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export declare function censor<F extends TypeLambda>(
  F: Functor<F>
): <W>(
  f: (w: W) => W
) => <S, R, FW, E, A>(self: Kind<F, S, R, FW, E, Writer<W, A>>) => Kind<F, S, R, FW, E, Writer<W, A>>
