/**
 * @since 3.0.0
 */
import type * as bounded from './Bounded'
import * as eq from './Eq'
import type * as field from './Field'
import type * as ord from './Ord'
import type * as show_ from './Show'
import type { Semigroup } from './Semigroup'
import type { Monoid } from './Monoid'
import type { Magma } from './Magma'
import type { Refinement } from './Refinement'
/**
 * @category refinements
 * @since 3.0.0
 */
export declare const isNumber: Refinement<unknown, number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Eq: eq.Eq<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Ord: ord.Ord<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bounded: bounded.Bounded<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Field: field.Field<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Show: show_.Show<number>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const MagmaSub: Magma<number>
/**
 * `number` semigroup under addition.
 *
 * @example
 * import { SemigroupSum } from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(2, SemigroupSum.combine(3)), 5)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupSum: Semigroup<number>
/**
 * `number` semigroup under multiplication.
 *
 * @example
 * import { SemigroupProduct } from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(2, SemigroupProduct.combine(3)), 6)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupProduct: Semigroup<number>
/**
 * `number` monoid under addition.
 *
 * The `empty` value is `0`.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const MonoidSum: Monoid<number>
/**
 * `number` monoid under multiplication.
 *
 * The `empty` value is `1`.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const MonoidProduct: Monoid<number>
